# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Kitti dataset."""

import collections
import csv
import os

import numpy as np
import tensorflow_datasets.public_api as tfds

_HOMEPAGE_URL = "http://www.cvlibs.net/datasets/kitti/"
_DATA_URL = "https://s3.eu-central-1.amazonaws.com/avg-kitti"
_IMAGES_FNAME = "data_object_image_2.zip"
_LABELS_FNAME = "data_object_label_2.zip"
_DEVKIT_FNAME = "devkit_object.zip"
_OBJECT_LABELS = [
    "Car",
    "Van",
    "Truck",
    "Pedestrian",
    "Person_sitting",
    "Cyclist",
    "Tram",
    "Misc",
]
_OCCLUDED_LABELS = [
    "fully visible",
    "partly occluded",
    "largely occluded",
    "unknown",
]
# The percentage of trainset videos to put into validation and test sets.
# The released test images do not have labels.
_VALIDATION_SPLIT_PERCENT_VIDEOS = 10
_TEST_SPLIT_PERCENT_VIDEOS = 10

# Raw Kitti representation of a bounding box. Coordinates are in pixels,
# measured from the top-left hand corner.
RawBoundingBox = collections.namedtuple(
    "RawBoundingBox", ["top", "bottom", "left", "right"]
)


class Builder(tfds.core.GeneratorBasedBuilder):
  """Kitti dataset."""

  VERSION = tfds.core.Version("3.3.0")
  SUPPORTED_VERSIONS = [
      tfds.core.Version("3.2.0"),
      tfds.core.Version("3.1.0"),
  ]
  RELEASE_NOTES = {
      "3.3.0": "Added labels for the `occluded` feature.",
      "3.2.0": "Devkit updated.",
  }

  def _info(self):
    # Annotation descriptions are in the object development kit.
    annotations = {
        "type": tfds.features.ClassLabel(
            names=_OBJECT_LABELS, doc="The type of object, e.g. 'Car' or 'Van'"
        ),
        "truncated": tfds.features.Tensor(
            shape=(),
            dtype=np.float32,
            doc=(
                "Float from 0 (non-truncated) to 1 (truncated), where"
                "truncated refers to the object leaving image boundaries"
            ),
        ),
        "occluded": tfds.features.ClassLabel(
            names=_OCCLUDED_LABELS,
            doc=(
                "Integer (0,1,2,3) indicating occlusion state: "
                "0 = fully visible, 1 = partly occluded"
                "2 = largely occluded, 3 = unknown"
            ),
        ),
        "alpha": tfds.features.Tensor(
            shape=(),
            dtype=np.float32,
            doc="Observation angle of object, ranging [-pi..pi]",
        ),
        "bbox": tfds.features.BBoxFeature(
            doc="2D bounding box of object in the image"
        ),
        "dimensions": tfds.features.Tensor(
            shape=(3,),
            dtype=np.float32,
            doc="3D object dimensions: height, width, length (in meters)",
        ),
        "location": tfds.features.Tensor(
            shape=(3,),
            dtype=np.float32,
            doc="3D object location x,y,z in camera coordinates (in meters)",
        ),
        "rotation_y": tfds.features.Tensor(
            shape=(),
            dtype=np.float32,
            doc="Rotation ry around Y-axis in camera coordinates [-pi..pi]",
        ),
    }
    return self.dataset_info_from_configs(
        features=tfds.features.FeaturesDict({
            "image": tfds.features.Image(),
            "image/file_name": tfds.features.Text(),  # E.g. "000001.png".
            "objects": tfds.features.Sequence(annotations),
        }),
        homepage=_HOMEPAGE_URL,
    )

  def _split_generators(self, dl_manager):
    filenames = {
        "images": _DATA_URL + "/" + _IMAGES_FNAME,
        "annotations": _DATA_URL + "/" + _LABELS_FNAME,
        "devkit": _DATA_URL + "/" + _DEVKIT_FNAME,
    }
    files = dl_manager.download(filenames)
    train_images, validation_images, test_images = _build_splits(
        dl_manager.iter_archive(files["devkit"])
    )

    return [
        tfds.core.SplitGenerator(
            name=tfds.Split.TRAIN,
            gen_kwargs={
                "images": dl_manager.iter_archive(files["images"]),
                "annotations": dl_manager.iter_archive(files["annotations"]),
                "subdir": "training",
                "image_ids": train_images,
            },
        ),
        tfds.core.SplitGenerator(
            name=tfds.Split.VALIDATION,
            gen_kwargs={
                "images": dl_manager.iter_archive(files["images"]),
                "annotations": dl_manager.iter_archive(files["annotations"]),
                "subdir": "training",
                "image_ids": validation_images,
            },
        ),
        tfds.core.SplitGenerator(
            name=tfds.Split.TEST,
            gen_kwargs={
                "images": dl_manager.iter_archive(files["images"]),
                "annotations": dl_manager.iter_archive(files["annotations"]),
                "subdir": "training",
                "image_ids": test_images,
            },
        ),
    ]

  def _generate_examples(self, images, annotations, subdir, image_ids):
    """Yields images and annotations.

    Args:
      images: object that iterates over the archive of images.
      annotations: object that iterates over the archive of annotations.
      subdir: subdirectory from which to extract images and annotations, e.g.
        training or testing.
      image_ids: file ids for images in this split.

    Yields:
      A tuple containing the example's key, and the example.
    """
    cv2 = tfds.core.lazy_imports.cv2

    all_annotations = dict()
    for fpath, fobj in annotations:
      prefix, ext = os.path.splitext(fpath)
      if ext != ".txt":
        continue
      if prefix.split(os.path.sep)[0] != subdir:
        continue

      # Key is the datapoint id. E.g. training/label_2/label_000016 -> 16.
      all_annotations[int(prefix[-6:])] = _parse_kitti_annotations(fobj)

    for fpath, fobj in images:
      prefix, ext = os.path.splitext(fpath)
      if ext != ".png":
        continue
      if prefix.split(os.path.sep)[0] != subdir:
        continue
      image_id = int(prefix[-6:])
      if image_id not in image_ids:
        continue
      annotations = all_annotations[image_id]
      img = cv2.imdecode(
          np.frombuffer(fobj.read(), dtype=np.uint8), cv2.IMREAD_COLOR
      )
      img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
      height, width, _ = img.shape
      for obj in annotations:
        obj["bbox"] = _build_bounding_box(obj["bbox_raw"], height, width)
        del obj["bbox_raw"]
      _, fname = os.path.split(fpath)
      record = {"image": img, "image/file_name": fname, "objects": annotations}
      yield fname, record


def _build_bounding_box(bbox, height, width):
  """Builds and returns TFDS bounding box.

  Args:
    bbox: RawBoundingBox, bounding box in Kitti coordinates (origin top left).
    height: Image height in pixels.
    width: Image width in pixels.

  Returns:
    A TFDS BBox (origin bottom left).
  """
  return tfds.features.BBox(
      ymin=(height - bbox.bottom) / height,
      ymax=(height - bbox.top) / height,
      xmin=bbox.left / width,
      xmax=bbox.right / width,
  )


def _parse_kitti_annotations(annotations_csv):
  """Loads and parses the Kitti object annotations.

  Args:
    annotations_csv: csv file containing annotations for a single image.

  Returns:
    A list of labelled bounding boxes. Each bounding box is stored as a
    dictionary of features.
  """
  annotations = []
  for line in annotations_csv:
    (
        obj_type,
        truncated,
        occluded,
        alpha,
        left,
        top,
        right,
        bottom,
        height,
        width,
        length,
        x,
        y,
        z,
        rotation_y,
    ) = list(csv.reader([line.decode()], delimiter=" "))[0]
    # DontCare objects lack annotations, so skip them.
    if obj_type == "DontCare":
      continue
    annotations.append({
        "type": obj_type,
        "truncated": float(truncated),
        "occluded": int(occluded),
        "alpha": float(alpha),
        "bbox_raw": RawBoundingBox(
            top=float(top),
            bottom=float(bottom),
            left=float(left),
            right=float(right),
        ),
        "dimensions": [float(v) for v in [height, width, length]],
        "location": [float(v) for v in [x, y, z]],
        "rotation_y": float(rotation_y),
    })
  return annotations


def _build_splits(devkit):
  """Splits the train data into train/val/test by video.

  Ensures that images from the same video do not traverse the splits.

  Args:
    devkit: object that iterates over the devkit archive.

  Returns:
    train_images: File ids for the training set images.
    validation_images: File ids for the validation set images.
    test_images: File ids for the test set images.
  """
  mapping_line_ids = None
  mapping_lines = None
  for fpath, fobj in devkit:
    if fpath == os.path.join("mapping", "train_rand.txt"):
      # Converts 1-based line index to 0-based line index.
      mapping_line_ids = [
          int(x.strip()) - 1 for x in fobj.read().decode("utf-8").split(",")
      ]
    elif fpath == os.path.join("mapping", "train_mapping.txt"):
      mapping_lines = fobj.read().splitlines()
      mapping_lines = [x.decode("utf-8") for x in mapping_lines]

  assert mapping_line_ids
  assert mapping_lines

  video_to_image = collections.defaultdict(list)
  for image_id, mapping_lineid in enumerate(mapping_line_ids):
    line = mapping_lines[mapping_lineid]
    video_id = line.split(" ")[1]
    video_to_image[video_id].append(image_id)

  # Sets numpy random state.
  numpy_original_state = np.random.get_state()
  np.random.seed(seed=123)

  # Max 1 for testing.
  num_test_videos = max(
      1, _TEST_SPLIT_PERCENT_VIDEOS * len(video_to_image) // 100
  )
  num_validation_videos = max(
      1, _VALIDATION_SPLIT_PERCENT_VIDEOS * len(video_to_image) // 100
  )
  test_videos = set(
      np.random.choice(
          sorted(list(video_to_image.keys())), num_test_videos, replace=False
      )
  )
  validation_videos = set(
      np.random.choice(
          sorted(list(set(video_to_image.keys()) - set(test_videos))),
          num_validation_videos,
          replace=False,
      )
  )
  test_images = []
  validation_images = []
  train_images = []
  for k, v in video_to_image.items():
    if k in test_videos:
      test_images.extend(v)
    elif k in validation_videos:
      validation_images.extend(v)
    else:
      train_images.extend(v)

  # Resets numpy random state.
  np.random.set_state(numpy_original_state)
  return train_images, validation_images, test_images



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Text feature."""

import html
import os
import textwrap
from typing import Union

from absl import logging
from etils import epath
import numpy as np
from tensorflow_datasets.core.deprecated import text as text_lib
from tensorflow_datasets.core.features import feature as feature_lib
from tensorflow_datasets.core.features import tensor_feature
from tensorflow_datasets.core.proto import feature_pb2
from tensorflow_datasets.core.utils import type_utils

Json = type_utils.Json


class Text(tensor_feature.Tensor):
  """`FeatureConnector` for text, encoding to integers with a `TextEncoder`."""

  def __init__(
      self,
      encoder=None,
      encoder_config=None,
      *,
      doc: feature_lib.DocArg = None,
      optional: bool = False,
  ):
    """Constructs a Text FeatureConnector.

    Args:
      encoder: `tfds.deprecated.text.TextEncoder`, an encoder that can convert
        text to integers. If None, the text will be utf-8 byte-encoded.
      encoder_config: `tfds.deprecated.text.TextEncoderConfig`, needed if
        restoring from a file with `load_metadata`.
      doc: Documentation of this feature (e.g. description).
      optional: Whether the feature is optional.
    """
    if encoder and encoder_config:
      raise ValueError("If encoder is provided, encoder_config must be None.")
    if encoder:
      encoder_config = text_lib.TextEncoderConfig(
          encoder_cls=type(encoder), vocab_size=encoder.vocab_size
      )
    elif encoder_config:
      encoder = encoder_config.encoder

    self._encoder = encoder
    self._encoder_config = encoder_config

    has_encoder = bool(encoder or self._encoder_cls)
    if has_encoder:
      logging.warning(
          "TFDS datasets with text encoding are deprecated and will be removed "
          "in a future version. Instead, you should use the plain text version "
          "and tokenize the text using `tensorflow_text` (See: "
          "https://www.tensorflow.org/tutorials/tensorflow_text/intro#tfdata_example)"
      )
    super(Text, self).__init__(
        shape=(None,) if has_encoder else (),
        dtype=np.int64 if has_encoder else np.object_,
        doc=doc,
        optional=optional,
    )

  @property
  def encoder(self):
    return self._encoder

  @encoder.setter
  def encoder(self, new_encoder):
    if self.encoder:
      raise ValueError("Cannot override encoder")
    self._encoder = new_encoder
    encoder_cls = self._encoder_cls or type(None)
    if not isinstance(new_encoder, encoder_cls):
      raise ValueError(
          "Changing type of encoder. Got %s but must be %s"
          % (type(new_encoder).__name__, self._encoder_cls.__name__)
      )

  def maybe_set_encoder(self, new_encoder):
    """Set encoder, but no-op if encoder is already set."""
    if self.encoder:
      return
    self.encoder = new_encoder

  @property
  def vocab_size(self):
    return self.encoder and self.encoder.vocab_size

  def str2ints(self, str_value):
    """Conversion string => encoded list[int]."""
    if not self._encoder:
      raise ValueError(
          "Text.str2ints is not available because encoder hasn't been defined."
      )
    return self._encoder.encode(str_value)

  def ints2str(self, int_values):
    """Conversion list[int] => decoded string."""
    if not self._encoder:
      raise ValueError(
          "Text.ints2str is not available because encoder hasn't been defined."
      )
    return self._encoder.decode(int_values)

  def encode_example(self, example_data):
    if self.encoder:
      example_data = self.encoder.encode(example_data)
    return super(Text, self).encode_example(example_data)

  def decode_example_np(self, example_data):
    return example_data

  def save_metadata(self, data_dir, feature_name: str) -> None:
    if not self.encoder:
      return
    fname_prefix = _file_name_prefix_for_metadata(feature_name, data_dir)
    self.encoder.save_to_file(fname_prefix)

  def load_metadata(self, data_dir, feature_name: str) -> None:
    if self._encoder_cls:
      fname_prefix = _file_name_prefix_for_metadata(feature_name, data_dir)
      self._encoder = self._encoder_cls.load_from_file(fname_prefix)  # pytype: disable=attribute-error
      return

    # Error checking: ensure there are no metadata files
    feature_files = list(epath.Path(data_dir).glob(f"{feature_name}.text*"))
    if feature_files:
      raise ValueError(
          "Text feature files found for feature %s but encoder_cls=None. "
          "Make sure to set encoder_cls in the TextEncoderConfig. "
          "Files: %s" % (feature_name, feature_files)
      )

  def maybe_build_from_corpus(self, corpus_generator, **kwargs):
    """Call SubwordTextEncoder.build_from_corpus is encoder_cls is such.

    If `self.encoder` is `None` and `self._encoder_cls` is of type
    `SubwordTextEncoder`, the method instantiates `self.encoder` as returned
    by `SubwordTextEncoder.build_from_corpus()`.

    Args:
      corpus_generator: generator yielding `str`, from which subwords will be
        constructed.
      **kwargs: kwargs forwarded to `SubwordTextEncoder.build_from_corpus()`
    """
    if self._encoder_cls is not text_lib.SubwordTextEncoder:
      return
    if self.encoder:
      return

    vocab_size = self._encoder_config.vocab_size
    self.encoder = text_lib.SubwordTextEncoder.build_from_corpus(
        corpus_generator=corpus_generator,
        target_vocab_size=vocab_size,
        **kwargs,
    )

  @property
  def _encoder_cls(self):
    return self._encoder_config and self._encoder_config.encoder_cls

  def _additional_repr_info(self):
    if self.encoder is None:
      return {}
    return {"encoder": repr(self.encoder)}

  def repr_html(self, ex: bytes) -> str:
    """Text are decoded."""
    if self.encoder is not None:
      return repr(ex)

    try:
      ex = ex.decode("utf-8")
    except UnicodeDecodeError:
      # Some datasets have invalid UTF-8 examples (e.g. opinosis)
      return repr(ex[:1000])
    ex = html.escape(ex)
    ex = textwrap.shorten(ex, width=1000)  # Truncate long text
    return ex

  @classmethod
  def from_json_content(
      cls, value: Union[Json, feature_pb2.TextFeature]
  ) -> "Text":
    if isinstance(value, dict) and "use_encoder" in value:
      raise ValueError(
          "Deprecated encoder not supported. Please use the plain text version "
          "with `tensorflow_text`."
      )
    return cls()

  def to_json_content(self) -> Union[Json, feature_pb2.TextFeature]:  # pytype: disable=signature-mismatch  # overriding-return-type-checks
    if self._encoder:
      logging.warning(
          "Dataset is using deprecated text encoder API which will be removed "
          "soon. Please use the plain_text version of the dataset and migrate "
          "to `tensorflow_text`."
      )
      return dict(use_encoder=True)
    return feature_pb2.TextFeature()


def _file_name_prefix_for_metadata(feature_name: str, data_dir) -> str:
  return os.path.join(data_dir, f"{feature_name}.text")



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Single location for all constants related to the simulation and RL environment.

Copyright 2022 Google LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    https://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

# --------- Thermal Constants ---------------

AIR_HEAT_CAPACITY = 1006.0  # J/kg/K, standard atmosphere
WATER_HEAT_CAPACITY = 4180.0  # J/kg/K
WATER_VAPOR_HEAT_CAPACITY = 1863.8  # J/kg/K

# --------- Energy Constants ---------------
BTU_PER_KWH: float = 3412.4
JOULES_PER_KWH: float = 3.6e6
JOULES_PER_BTU: float = 1055.06
W_PER_KW: float = 1000.0  # Number of Watts in a kW.
WATTS_PER_BTU_HR: float = 0.29307107  # Number of Watts in a BTU/hr
HZ_PERCENT: float = 100.0 / 60.0  # Converts blower/pump Hz to Percentage Power

# https://www.rapidtables.com/convert/power/hp-to-watt.html
WATTS_PER_HORSEPOWER = 746.0

# Natural gas energy conversion.
# Sources:
# http://www.kylesconverter.com/energy,-work,-and-heat/kilowatt--hours-to-cubic-feet-of-natural-gas#293.07
# https://www.traditionaloven.com/tutorials/energy/convert-cubic-foot-natural-gas-to-kilo-watt-hr-kwh.html
KWH_PER_KFT3_GAS = 293.07107  # kWh / 1000 cubic feet natural gas

# Natural gas CO2 emission.
# Source: https://www.eia.gov/environment/emissions/co2_vol_mass.php
GAS_CO2 = 53.12  # kg/1000 cubic feet

WATER_DENSITY = 1000.0  # kg/m3
GRAVITY = 9.8  # m/s2

# ------------ File Name Convention -----------------
NORMALIZATION_FILENAME = 'normalization_info'
OBSERVATION_RESPONSE_FILE_PREFIX = 'observation_response'
ACTION_RESPONSE_FILE_PREFIX = 'action_response'
IMAGES_DIR = 'images'
BUILDING_IMAGE_CSV_FILE = 'building_images.csv'
REWARD_INFO_PREFIX = 'reward_info'
REWARD_RESPONSE_PREFIX = 'reward_response'
DEVICE_INFO_PREFIX = 'device_info'
ZONE_INFO_PREFIX = 'zone_info'



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Reddit TIFU dataset using tifu or tldr from subreddit tifu."""

import json

import numpy as np
from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
import tensorflow_datasets.public_api as tfds

_URL = "https://drive.google.com/uc?export=download&id=1ffWfITKFMJeqjT8loC8aiCLRNJpc_XnF"
_LONG_SPLIT = "https://storage.googleapis.com/tfds-data/downloads/reddit_tifu/reddit_tifu_long_splits.json"

_DOCUMENT = "documents"
_TITLE = "title"
_TLDR = "tldr"
_ID = "id"
_ADDITIONAL_FEATURES = ["ups", "num_comments", "score", "upvote_ratio"]


class RedditTifuConfig(tfds.core.BuilderConfig):
  """BuilderConfig for RedditTifu."""

  def __init__(self, *, summary_key=None, **kwargs):
    """BuilderConfig for RedditTifu.

    Args:
      summary_key: key string of summary in downloaded json file.
      **kwargs: keyword arguments forwarded to super.
    """
    super(RedditTifuConfig, self).__init__(
        version=tfds.core.Version("1.1.2"), **kwargs
    )
    self.summary_key = summary_key


class Builder(tfds.core.GeneratorBasedBuilder):
  """Reddit TIFU Dataset."""

  RELEASE_NOTES = {
      "1.1.0": "Remove empty document and summary strings.",
      "1.1.1": (
          "Add train, dev and test (80/10/10) splits which are used in "
          "PEGASUS (https://arxiv.org/abs/1912.08777) in a separate config. "
          "These were created randomly using the tfds split function and are "
          "being released to ensure that results on Reddit Tifu Long are "
          "reproducible and comparable."
          "Also add `id` to the datapoints."
      ),
      "1.1.2": "Corrected splits uploaded.",
  }
  BUILDER_CONFIGS = [
      RedditTifuConfig(
          name="short",
          summary_key=_TITLE,
          description="Using title as summary.",
      ),
      RedditTifuConfig(
          name="long",
          summary_key=_TLDR,
          description="Using TLDR as summary.",
      ),
      RedditTifuConfig(
          name="long_split",
          summary_key=_TLDR,
          description="Using TLDR as summary and return train/test/dev splits.",
      ),
  ]

  def _info(self):
    features = {
        k: tfds.features.Tensor(shape=[], dtype=np.float32)
        for k in _ADDITIONAL_FEATURES
    }
    features.update(
        {k: tfds.features.Text() for k in [_DOCUMENT, _TLDR, _TITLE, _ID]}
    )
    return self.dataset_info_from_configs(
        features=tfds.features.FeaturesDict(features),
        supervised_keys=(_DOCUMENT, self.builder_config.summary_key),
        homepage="https://github.com/ctr4si/MMN",
    )

  def _split_generators(self, dl_manager):
    """Returns SplitGenerators."""
    if self.builder_config.name == "long_split":
      paths = dl_manager.download_and_extract(
          {"data": _URL, "split": _LONG_SPLIT}
      )
      dl_path = paths["data"]
      split_path = paths["split"]
      return [
          tfds.core.SplitGenerator(
              name=tfds.Split.TRAIN,
              gen_kwargs={
                  "path": dl_path,
                  "split_path": split_path,
                  "split": "train",
              },
          ),
          tfds.core.SplitGenerator(
              name=tfds.Split.TEST,
              gen_kwargs={
                  "path": dl_path,
                  "split_path": split_path,
                  "split": "test",
              },
          ),
          tfds.core.SplitGenerator(
              name=tfds.Split.VALIDATION,
              gen_kwargs={
                  "path": dl_path,
                  "split_path": split_path,
                  "split": "validation",
              },
          ),
      ]
    else:
      dl_path = dl_manager.download_and_extract(_URL)
      return [
          tfds.core.SplitGenerator(
              name=tfds.Split.TRAIN,
              gen_kwargs={"path": dl_path},
          )
      ]

  def _generate_examples(self, path=None, split_path=None, split=None):
    """Yields examples."""
    if split_path:
      split_data = json.load(tf.io.gfile.GFile(split_path))
    with tf.io.gfile.GFile(path, "rb") as f:
      for i, line in enumerate(f):
        # keys are 'title_tokenized','permalink','title','url','num_comments',
        #   'tldr'(optional),'created_utc','trimmed_title_tokenized','ups',
        #   'selftext_html','score','upvote_ratio','tldr_tokenized'(optional),
        #   'selftext','trimmed_title','selftext_without_tldr_tokenized',
        #   'id','selftext_without_tldr'
        d = json.loads(line)
        if not split or (split and d["id"] in split_data[split]):
          r = {
              _DOCUMENT: d["selftext_without_tldr"].strip(),
              _TITLE: d["trimmed_title"].strip(),
              _TLDR: (d["tldr"] or "").strip(),
              _ID: d["id"].strip(),
          }
          r.update({k: d[k] for k in _ADDITIONAL_FEATURES})
          # skip if document or summary is empty
          if r[_DOCUMENT] and r[self.builder_config.summary_key]:
            yield i, r



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Natural Questions: A Benchmark for Question Answering Research."""
import html
import json
import os
import re
from typing import Optional

import numpy as np
import tensorflow_datasets.public_api as tfds

_BEAM_NAMESPACE = 'TFDS_LONGT5'

_URL = 'https://ai.google.com/research/NaturalQuestions/dataset'

_BASE_DOWNLOAD_URL = 'https://storage.googleapis.com/natural_questions/v1.0'
_DOWNLOAD_URLS = {
    'train': [
        '%s/train/nq-train-%02d.jsonl.gz' % (_BASE_DOWNLOAD_URL, i)
        for i in range(50)
    ],
    'validation': [
        '%s/dev/nq-dev-%02d.jsonl.gz' % (_BASE_DOWNLOAD_URL, i)
        for i in range(5)
    ],
}

_CITATIONS = {
    'longt5': """@misc{guo2021longt5,
        title={LongT5: Efficient Text-To-Text Transformer for Long Sequences},
        author={Mandy Guo and Joshua Ainslie and David Uthus and Santiago Ontanon and Jianmo Ni and Yun-Hsuan Sung and Yinfei Yang},
        year={2021},
        eprint={2112.07916},
        archivePrefix={arXiv},
        primaryClass={cs.CL}
        }
    """,
}


def _features():
  return {
      'default': tfds.features.FeaturesDict({
          'id': np.str_,
          'document': {
              'title': tfds.features.Text(),
              'url': tfds.features.Text(),
              'html': tfds.features.Text(),
              'tokens': tfds.features.Sequence({
                  'token': tfds.features.Text(),
                  'is_html': np.bool_,
              }),
          },
          'question': {
              'text': tfds.features.Text(),
              'tokens': tfds.features.Sequence(np.str_),
          },
          'annotations': tfds.features.Sequence({
              'id': np.str_,
              'long_answer': {
                  'start_token': np.int64,
                  'end_token': np.int64,
                  'start_byte': np.int64,
                  'end_byte': np.int64,
              },
              'short_answers': tfds.features.Sequence({
                  'start_token': np.int64,
                  'end_token': np.int64,
                  'start_byte': np.int64,
                  'end_byte': np.int64,
                  'text': tfds.features.Text(),
              }),
              'yes_no_answer': tfds.features.ClassLabel(
                  names=['NO', 'YES']
              ),  # Can also be -1 for NONE.
          }),
      }),
      'longt5': tfds.features.FeaturesDict({
          'id': tfds.features.Text(),
          'title': tfds.features.Text(),
          'context': tfds.features.Text(),
          'question': tfds.features.Text(),
          'answer': tfds.features.Text(),
          'all_answers': tfds.features.Sequence(tfds.features.Text()),
      }),
  }


class NaturalQuestionsConfig(tfds.core.BuilderConfig):
  """NaturalQuestions for Longt5."""

  def __init__(
      self,
      *,
      citation: Optional[str] = None,
      features: Optional[tfds.features.FeaturesDict] = None,
      **kwargs,
  ):
    """BuilderConfig for Longt5.

    Args:
      citation: `string`, citation for the config.
      features: `dict[string, string]`, map from the name of the feature dict
        for each text field to the name of the column in the tsv file.
      **kwargs: keyword arguments forwarded to super.
    """
    super(NaturalQuestionsConfig, self).__init__(**kwargs)
    self.citation = citation
    self.features = features


class Builder(tfds.core.BeamBasedBuilder):
  """Natural Questions: A Benchmark for Question Answering Research."""

  VERSION = tfds.core.Version('0.1.0')
  SUPPORTED_VERSIONS = [tfds.core.Version('0.0.2'), tfds.core.Version('0.1.0')]
  BUILDER_CONFIGS = [
      NaturalQuestionsConfig(
          name='default',
          description='Default natural_questions config',
          features=_features()['default'],
      ),
      NaturalQuestionsConfig(
          name='longt5',
          description=(
              'natural_questions preprocessed as in the longT5 benchmark'
          ),
          citation=_CITATIONS['longt5'],
          features=_features()['longt5'],
      ),
  ]
  DEFAULT_CONFIG_NAME = 'default'

  def _info(self):
    return self.dataset_info_from_configs(
        features=self.builder_config.features,
        supervised_keys=None,
        homepage=_URL,
    )

  def _split_generators(self, dl_manager):
    """Returns SplitGenerators."""

    files = dl_manager.download(_DOWNLOAD_URLS)

    return [
        tfds.core.SplitGenerator(
            name=tfds.Split.TRAIN,
            gen_kwargs={'filepaths': files['train']},
        ),
        tfds.core.SplitGenerator(
            name=tfds.Split.VALIDATION,
            gen_kwargs={'filepaths': files['validation']},
        ),
    ]

  @staticmethod
  def _parse_example_default(line):
    """Parse a single json line and emit an example dict."""

    def _parse_short_answer_default(html_bytes, short_ans):
      """Extract text of short answer."""
      ans_bytes = html_bytes[short_ans['start_byte'] : short_ans['end_byte']]
      # Remove non-breaking spaces.
      ans_bytes = ans_bytes.replace(b'\xc2\xa0', b' ')
      text = ans_bytes.decode('utf-8')
      # Remove HTML markup.
      text = re.sub('<([^>]*)>', '', html.unescape(text))
      # Replace \xa0 characters with spaces.
      return {
          'start_token': short_ans['start_token'],
          'end_token': short_ans['end_token'],
          'start_byte': short_ans['start_byte'],
          'end_byte': short_ans['end_byte'],
          'text': text,
      }

    def _parse_annotation_default(html_bytes, an_json):
      return {
          # Convert to str since some IDs cannot be represented by np.int64.
          'id': str(an_json['annotation_id']),
          'long_answer': {
              'start_token': an_json['long_answer']['start_token'],
              'end_token': an_json['long_answer']['end_token'],
              'start_byte': an_json['long_answer']['start_byte'],
              'end_byte': an_json['long_answer']['end_byte'],
          },
          'short_answers': [
              _parse_short_answer_default(html_bytes, ans)
              for ans in an_json['short_answers']
          ],
          'yes_no_answer': (
              -1
              if an_json['yes_no_answer'] == 'NONE'
              else an_json['yes_no_answer']
          ),
      }

    ex_json = json.loads(line)
    html_bytes = ex_json['document_html'].encode('utf-8')

    # Convert to str since some IDs cannot be represented by np.int64.
    id_ = str(ex_json['example_id'])
    return id_, {
        'id': id_,
        'document': {
            'title': ex_json['document_title'],
            'url': ex_json['document_url'],
            'html': html_bytes,
            'tokens': [
                {  # pylint: disable=g-complex-comprehension
                    'token': t['token'],
                    'is_html': t['html_token'],
                }
                for t in ex_json['document_tokens']
            ],
        },
        'question': {
            'text': ex_json['question_text'],
            'tokens': ex_json['question_tokens'],
        },
        'annotations': [
            _parse_annotation_default(html_bytes, an_json)
            for an_json in ex_json['annotations']
        ],
    }

  @staticmethod
  def _parse_example_longt5(line):
    """Parse a single json line and emit an example dict."""
    ex_json = json.loads(line)

    def _pad_punctuation(text):
      """Adds spaces around punctuation."""
      # Add space around punctuation.
      text = re.sub(r'(\W)', r' \1 ', text)
      # Collapse consecutive whitespace into one space.
      text = re.sub(r'\s+', ' ', text)
      return text

    def _get_context(json_dict):
      document_tokens = []
      for c in json_dict['long_answer_candidates']:
        if not bool(c['top_level']):
          continue
        for pos in range(c['start_token'], c['end_token']):
          t = json_dict['document_tokens'][pos]
          document_tokens.append(t['token'])
      context = ' '.join(document_tokens)
      # Substitute newlines and tabs with spaces.
      context = re.sub(r'\n\t', ' ', context)
      context = _pad_punctuation(context)
      return context

    def _has_short_answer(a):
      return bool(a['short_answers'])

    def _is_yes_no_answer(a):
      return a['yes_no_answer'] in ('YES', 'NO')

    def _get_all_answers(json_dict):
      all_answers = []
      for a in json_dict['annotations']:
        answer_tokens = []
        if _has_short_answer(a):
          for sa in a['short_answers']:
            for pos in range(sa['start_token'], sa['end_token']):
              answer_tokens.append(
                  _pad_punctuation(json_dict['document_tokens'][pos]['token'])
              )
            answer = ' '.join(answer_tokens)
            all_answers.append(answer)
        elif _is_yes_no_answer(a):
          all_answers.append(_pad_punctuation(a['yes_no_answer']))
        else:
          all_answers.append(_pad_punctuation('NULL'))
      return all_answers

    def _get_answer(all_answers):
      """Pick the first non NULL answer, if it exists."""
      for a in all_answers:
        if a != 'NULL':
          return a
      return 'NULL'

    # Convert to str since some IDs cannot be represented by np.int64.
    id_ = str(ex_json['example_id'])
    all_answers = _get_all_answers(ex_json)
    return id_, {
        'id': id_,
        'question': _pad_punctuation(ex_json['question_text'].strip()),
        'title': ex_json['document_title'].strip(),
        'context': _get_context(ex_json),
        'all_answers': all_answers,
        'answer': _get_answer(all_answers),
    }

  def _build_pcollection(self, pipeline, filepaths):
    """Build PCollection of examples."""
    beam = tfds.core.lazy_imports.apache_beam

    parse_example = {
        'default': self._parse_example_default,
        'longt5': self._parse_example_longt5,
    }[self.builder_config.name]

    return (
        pipeline
        | beam.Create([os.fspath(f) for f in filepaths])
        | beam.io.ReadAllFromText()
        | beam.Map(parse_example)
    )



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Street View House Numbers (SVHN) Dataset, cropped version."""
import urllib

import numpy as np
from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
import tensorflow_datasets.public_api as tfds

URL = "http://ufldl.stanford.edu/housenumbers/"


class Builder(tfds.core.GeneratorBasedBuilder):
  """Street View House Numbers (SVHN) Dataset, cropped version."""

  VERSION = tfds.core.Version("3.1.0")
  SUPPORTED_VERSIONS = [
      tfds.core.Version("3.1.0"),
  ]
  RELEASE_NOTES = {
      "3.1.0": "New split API (https://tensorflow.org/datasets/splits)",
  }

  def _info(self):
    features_dict = {
        "image": tfds.features.Image(shape=(32, 32, 3)),
        "label": tfds.features.ClassLabel(num_classes=10),
    }
    if self.version > "3.0.0":
      features_dict["id"] = tfds.features.Text()
    return self.dataset_info_from_configs(
        description=(
            "The Street View House Numbers (SVHN) Dataset is an image digit "
            "recognition dataset of over 600,000 digit images coming from "
            "real world data. Images are cropped to 32x32."
        ),
        features=tfds.features.FeaturesDict(features_dict),
        supervised_keys=("image", "label"),
        homepage=URL,
    )

  def _split_generators(self, dl_manager):
    output_files = dl_manager.download({
        "train": urllib.parse.urljoin(URL, "train_32x32.mat"),
        "test": urllib.parse.urljoin(URL, "test_32x32.mat"),
        "extra": urllib.parse.urljoin(URL, "extra_32x32.mat"),
    })

    return [
        tfds.core.SplitGenerator(
            name=tfds.Split.TRAIN,
            gen_kwargs=dict(
                split_prefix="train_",
                filepath=output_files["train"],
            ),
        ),
        tfds.core.SplitGenerator(
            name=tfds.Split.TEST,
            gen_kwargs=dict(
                split_prefix="test_",
                filepath=output_files["test"],
            ),
        ),
        tfds.core.SplitGenerator(
            name="extra",
            gen_kwargs=dict(
                split_prefix="extra_",
                filepath=output_files["extra"],
            ),
        ),
    ]

  def _generate_examples(self, split_prefix, filepath):
    """Generate examples as dicts.

    Args:
      split_prefix: `str` prefix that identifies the split.
      filepath: `str` path of the file to process.

    Yields:
      Generator yielding the next samples
    """
    with tf.io.gfile.GFile(filepath, "rb") as f:
      data = tfds.core.lazy_imports.scipy.io.loadmat(f)

    # Maybe should shuffle ?

    assert np.max(data["y"]) <= 10  # Sanity check
    assert np.min(data["y"]) > 0

    for i, (image, label) in enumerate(
        zip(np.rollaxis(data["X"], -1), data["y"])
    ):
      label = label.reshape(())
      record = {
          "image": image,
          "label": label % 10,  # digit 0 is saved as 0 (instead of 10)
      }
      if self.version > "3.0.0":
        record["id"] = "{}{:06d}".format(split_prefix, i)
      yield i, record


# TODO(tfds): Add the SvhnFull dataset



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for all data sources."""

import pickle
from unittest import mock

import cloudpickle
from etils import epath
import pytest
import tensorflow_datasets as tfds
from tensorflow_datasets import testing
from tensorflow_datasets.core import dataset_builder
from tensorflow_datasets.core import dataset_info as dataset_info_lib
from tensorflow_datasets.core import decode
from tensorflow_datasets.core import file_adapters
from tensorflow_datasets.core import splits as splits_lib
from tensorflow_datasets.core.data_sources import array_record
from tensorflow_datasets.core.data_sources import parquet
from tensorflow_datasets.core.utils import shard_utils
from tensorflow_datasets.core.utils.lazy_imports_utils import array_record_data_source
from tensorflow_datasets.core.utils.lazy_imports_utils import parquet as pq


@pytest.fixture()
def mocked_array_record_dataset():
  mock_data_source = mock.create_autospec(
      array_record_data_source.ArrayRecordDataSource,
      spec_set=True,
  )
  with mock.patch.object(
      array_record_data_source,
      'ArrayRecordDataSource',
      return_value=mock_data_source,
  ):
    yield


@pytest.fixture()
def mocked_parquet_dataset():
  mock_data_source = mock.create_autospec(
      pq.ParquetDataset,
      spec_set=True,
  )
  with mock.patch.object(
      pq,
      'ParquetDataset',
      return_value=mock_data_source,
  ):
    yield


_FIXTURES = ['mocked_array_record_dataset', 'mocked_parquet_dataset']
_DATA_SOURCE_CLS = [
    array_record.ArrayRecordDataSource,
    parquet.ParquetDataSource,
]


@pytest.mark.parametrize(
    'file_format', file_adapters.FileFormat.with_random_access()
)
@pytest.mark.parametrize(
    'builder_cls',
    [testing.DummyDataset, testing.DummyBeamDataset],
)
def test_read_write(
    tmp_path: epath.Path,
    builder_cls: dataset_builder.DatasetBuilder,
    file_format: file_adapters.FileFormat,
):
  builder = builder_cls(data_dir=tmp_path, file_format=file_format)
  builder.download_and_prepare()
  data_source = builder.as_data_source(split='train')
  assert len(data_source) == 3
  for i in range(3):
    assert data_source[i] == {'id': i}
  assert data_source.__getitems__([0, 2]) == [{'id': 0}, {'id': 2}]
  assert data_source.__getitems__(range(0, 2)) == [{'id': 0}, {'id': 1}]
  assert data_source.__getitems__([]) == []  # pylint: disable=g-explicit-bool-comparison
  for i, element in enumerate(data_source):
    assert element == {'id': i}


_FILE_INSTRUCTIONS = [
    shard_utils.FileInstruction(
        'my_file-000-of-003', skip=0, take=12, examples_in_shard=12
    ),
    shard_utils.FileInstruction(
        'my_file-001-of-003', skip=2, take=9, examples_in_shard=11
    ),
    shard_utils.FileInstruction(
        'my_file-002-of-003', skip=0, take=4, examples_in_shard=4
    ),
]


def create_dataset_info(file_format: file_adapters.FileFormat):
  with mock.patch.object(splits_lib, 'SplitInfo') as split_mock:
    split_mock.return_value.name = 'train'
    split_mock.return_value.file_instructions = _FILE_INSTRUCTIONS
    dataset_info = mock.create_autospec(dataset_info_lib.DatasetInfo)
    dataset_info.file_format = file_format
    dataset_info.splits = {'train': split_mock()}
    dataset_info.name = 'dataset_name'
    return dataset_info


@pytest.mark.parametrize(
    'data_source_cls',
    _DATA_SOURCE_CLS,
)
def test_missing_split_raises_error(data_source_cls):
  dataset_info = create_dataset_info(file_adapters.FileFormat.ARRAY_RECORD)
  with pytest.raises(
      ValueError,
      match="Unknown split 'doesnotexist'.",
  ):
    data_source_cls(dataset_info, split='doesnotexist')


@pytest.mark.usefixtures(*_FIXTURES)
@pytest.mark.parametrize(
    'data_source_cls',
    _DATA_SOURCE_CLS,
)
def test_repr_returns_meaningful_string_without_decoders(data_source_cls):
  dataset_info = create_dataset_info(file_adapters.FileFormat.ARRAY_RECORD)
  source = data_source_cls(dataset_info, split='train')
  name = data_source_cls.__name__
  assert (
      repr(source) == f"{name}(name=dataset_name, split='train', decoders=None)"
  )


@pytest.mark.usefixtures(*_FIXTURES)
@pytest.mark.parametrize(
    'data_source_cls',
    _DATA_SOURCE_CLS,
)
def test_repr_returns_meaningful_string_with_decoders(data_source_cls):
  dataset_info = create_dataset_info(file_adapters.FileFormat.ARRAY_RECORD)
  source = data_source_cls(
      dataset_info,
      split='train',
      decoders={'my_feature': decode.SkipDecoding()},
  )
  name = data_source_cls.__name__
  assert (
      repr(source)
      == f'{name}(name=dataset_name,'
      " split='train', decoders={'my_feature': <class"
      " 'tensorflow_datasets.core.decode.base.SkipDecoding'>})"
  )


def test_data_source_is_sliceable():
  mock_array_record_data_source = tfds.testing.PickableDataSourceMock()
  with tfds.testing.mock_data(
      mock_array_record_data_source=mock_array_record_data_source
  ):
    tfds.data_source('mnist', split='train')
    assert len(mock_array_record_data_source.call_args_list) == 1
    file_instructions = mock_array_record_data_source.call_args_list[0].args[0]
    assert file_instructions[0].skip == 0
    assert file_instructions[0].take == 60000

    tfds.data_source('mnist', split='train[:50%]')
    assert len(mock_array_record_data_source.call_args_list) == 2
    file_instructions = mock_array_record_data_source.call_args_list[1].args[0]
    assert file_instructions[0].skip == 0
    assert file_instructions[0].take == 30000


# PyGrain requires that data sources are picklable.
@pytest.mark.parametrize(
    'file_format',
    file_adapters.FileFormat.with_random_access(),
)
@pytest.mark.parametrize('pickle_module', [pickle, cloudpickle])
def test_data_source_is_picklable_after_use(file_format, pickle_module):
  with tfds.testing.tmp_dir() as data_dir:
    builder = tfds.testing.DummyDataset(data_dir=data_dir)
    builder.download_and_prepare(file_format=file_format)
    data_source = builder.as_data_source(split='train')
    assert data_source[0] == {'id': 0}
    assert pickle_module.loads(pickle_module.dumps(data_source))[0] == {'id': 0}



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

r"""`tfds convert_format` command.

Example usage:
```
tfds convert_format \
  --dataset_version_dir=/data/dataset/config/1.2.3 \
  --out_file_format=array_record \
  --out_dir=/data_array_record/dataset/config/1.2.3 \
  --use_beam=True
```
"""

import dataclasses

from etils import epath
import simple_parsing
from tensorflow_datasets.core import file_adapters
from tensorflow_datasets.scripts.cli import cli_utils
from tensorflow_datasets.scripts.cli import convert_format_utils


@dataclasses.dataclass(frozen=True, kw_only=True)
class Args(cli_utils.Args):
  """Converts a dataset from one file format to another format.

  Attributes:
    root_data_dir: Root data dir that contains all datasets. All datasets and
      all their configs and versions that are in this folder will be converted.
    dataset_dir: Path where the dataset to be converted is located. Converts all
      configs and versions in this folder.
    dataset_version_dir: Path where the dataset to be converted is located.
      Should include config and version. Can also be a comma-separated list of
      paths. If multiple paths are specified, `--out_dir` should not be
      specified, since each dataset will be converted in the same directory as
      the input dataset.
    out_file_format: File format to convert the dataset to.
    out_dir: Path where the converted dataset will be stored. Datasets will be
      stored with the same folder structure as the input folder. If `None`, the
      converted shards will be stored in the same folder as the input datasets.
    overwrite: Whether to overwrite the output directory if it already exists.
    use_beam: Use beam to convert the dataset.
    num_workers: Number of workers to use when not using Beam. If `--use_beam`
      is set, this flag is ignored. If `--num_workers=1`, the conversion will be
      done sequentially.
    only_log_errors: If set, errors during the conversion will be logged as
      errors and will not crash the conversion. If you are converting a large
      number of datasets, you might want to set this flag to true.
  """

  root_data_dir: epath.Path | None = None
  dataset_dir: epath.Path | None = None
  dataset_version_dir: list[epath.Path] = simple_parsing.field(
      default_factory=list,
      type=lambda dataset_version_dirs_str: [
          epath.Path(path) for path in dataset_version_dirs_str.split(',')
      ],
      nargs='?',
  )
  out_file_format: str = simple_parsing.choice(
      *(file_format.value for file_format in file_adapters.FileFormat),
  )
  out_dir: epath.Path | None = None
  overwrite: bool = False
  use_beam: bool = False
  num_workers: int = 8
  only_log_errors: bool = False

  def execute(self) -> None:
    """Converts a dataset from one file format to another."""
    convert_format_utils.convert_dataset(
        out_dir=self.out_dir,
        out_file_format=self.out_file_format,
        dataset_dir=self.dataset_dir,
        root_data_dir=self.root_data_dir,
        dataset_version_dir=self.dataset_version_dir,
        overwrite=self.overwrite,
        use_beam=self.use_beam,
        num_workers=self.num_workers,
        fail_on_error=not self.only_log_errors,
    )



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import collections

from tensorflow_datasets.core import dataset_utils
from tensorflow_datasets.datasets.imagenet_a import imagenet_a_dataset_builder
import tensorflow_datasets.public_api as tfds


class ImagenetATest(tfds.testing.DatasetBuilderTestCase):
  DATASET_CLASS = imagenet_a_dataset_builder.Builder
  SPLITS = {
      'test': 10,  # Number of fake test examples.
  }

  DL_EXTRACT_RESULT = 'imagenet-a.tar'

  def _assertAsDataset(self, builder):
    """Check the label distribution.

    This checks that lable get correctly converted between the synset ids
    and integers.

    Args:
      builder: The ImagenetA dataset builder.
    """
    super()._assertAsDataset(builder)
    label_frequncies = collections.Counter()
    label_feature = builder.info.features['label']
    dataset = builder.as_dataset()
    for features in dataset_utils.as_numpy(dataset['test']):
      label_frequncies.update([label_feature.int2str(features['label'])])
    self.assertEqual(
        dict(label_frequncies), {'n01580077': 2, 'n01616318': 3, 'n07697313': 5}
    )


if __name__ == '__main__':
  tfds.testing.test_main()



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Frames Labeled In Cinema (FLIC)."""

import os

import numpy as np
from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
import tensorflow_datasets.public_api as tfds

_DATA_OPTIONS = ["small", "full"]

_HOMEPAGE_URL = "https://bensapp.github.io/flic-dataset.html"

_URL_SUBSET = "https://drive.google.com/uc?id=0B4K3PZp8xXDJN0Fpb0piVjQ3Y3M&export=download"
_URL_SUPERSET = "https://drive.google.com/uc?id=0B4K3PZp8xXDJd2VwblhhOVBfMDg&export=download"


def _normalize_bbox(raw_bbox, img_path):
  """Normalize torsobox bbox values."""
  with tf.io.gfile.GFile(img_path, "rb") as fp:
    img = tfds.core.lazy_imports.PIL_Image.open(fp)
    width, height = img.size

  return tfds.features.BBox(
      ymin=raw_bbox[1] / height,
      ymax=raw_bbox[3] / height,
      xmin=raw_bbox[0] / width,
      xmax=raw_bbox[2] / width,
  )


class FlicConfig(tfds.core.BuilderConfig):
  """BuilderConfig for FLIC."""

  def __init__(self, *, data, **kwargs):
    """Constructs a FlicConfig."""
    if data not in _DATA_OPTIONS:
      raise ValueError("data must be one of %s" % _DATA_OPTIONS)

    descriptions = {
        "small": "5003 examples used in CVPR13 MODEC paper.",
        "full": (
            "20928 examples, a superset of FLIC consisting of more difficult "
            "examples."
        ),
    }
    description = kwargs.get("description", "Uses %s" % descriptions[data])
    kwargs["description"] = description

    super(FlicConfig, self).__init__(**kwargs)
    self.data = data
    self.url = _URL_SUBSET if data == "small" else _URL_SUPERSET
    self.dir = "FLIC" if data == "small" else "FLIC-full"


def _make_builder_configs():
  configs = []
  for data in _DATA_OPTIONS:
    configs.append(
        FlicConfig(name=data, version=tfds.core.Version("2.0.0"), data=data)
    )
  return configs


class Builder(tfds.core.GeneratorBasedBuilder):
  """Frames Labeled In Cinema (FLIC)."""

  BUILDER_CONFIGS = _make_builder_configs()

  def _info(self):
    return self.dataset_info_from_configs(
        features=tfds.features.FeaturesDict({
            "image": tfds.features.Image(
                shape=(480, 720, 3), encoding_format="jpeg"
            ),
            "poselet_hit_idx": tfds.features.Sequence(np.uint16),
            "moviename": tfds.features.Text(),
            "xcoords": tfds.features.Sequence(np.float64),
            "ycoords": tfds.features.Sequence(np.float64),
            "currframe": tfds.features.Tensor(shape=(), dtype=np.float64),
            "torsobox": tfds.features.BBoxFeature(),
        }),
        homepage=_HOMEPAGE_URL,
    )

  def _split_generators(self, dl_manager):
    """Returns SplitGenerators."""
    extract_path = dl_manager.download_and_extract(self.builder_config.url)

    mat_path = os.path.join(
        extract_path, self.builder_config.dir, "examples.mat"
    )
    with tf.io.gfile.GFile(mat_path, "rb") as f:
      data = tfds.core.lazy_imports.scipy.io.loadmat(
          f, struct_as_record=True, squeeze_me=True, mat_dtype=True
      )

    return [
        tfds.core.SplitGenerator(
            name=tfds.Split.TRAIN,
            gen_kwargs={
                "extract_path": extract_path,
                "data": data,
                "selection_column": 7,  # indicates train split selection
            },
        ),
        tfds.core.SplitGenerator(
            name=tfds.Split.TEST,
            gen_kwargs={
                "extract_path": extract_path,
                "data": data,
                "selection_column": 8,  # indicates test split selection
            },
        ),
    ]

  def _generate_examples(self, extract_path, data, selection_column):
    """Yields examples."""
    for u_id, example in enumerate(data["examples"]):
      if example[selection_column]:
        img_path = os.path.join(
            extract_path, self.builder_config.dir, "images", example[3]
        )
        yield u_id, {
            "image": img_path,
            "poselet_hit_idx": example[0],
            "moviename": example[1],
            "xcoords": example[2][0],
            "ycoords": example[2][1],
            "currframe": example[5],
            "torsobox": _normalize_bbox(example[6], img_path),
        }



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for registry."""

import os
import tempfile
import textwrap
from unittest import mock

from etils import epath
import pytest
from tensorflow_datasets import testing
from tensorflow_datasets.core import naming
from tensorflow_datasets.core import registered
from tensorflow_datasets.core.community import config as config_lib
from tensorflow_datasets.core.community import register_package
from tensorflow_datasets.core.community import register_path
from tensorflow_datasets.core.community import registry as registry_lib
from tensorflow_datasets.core.utils import gcs_utils


class Ds0(testing.DummyDataset):
  pass


class Ds1(testing.DummyDataset):
  pass


@pytest.fixture(scope='module')
def dummy_register():
  """Dummy register."""
  with tempfile.TemporaryDirectory() as tmp_path:
    tmp_path = epath.Path(tmp_path)

    # Prepare the datasets
    # Namespace 0
    Ds0(data_dir=tmp_path / 'kaggle').download_and_prepare()
    Ds1(data_dir=tmp_path / 'kaggle2').download_and_prepare()
    # Namespace 1
    Ds0(data_dir=tmp_path / 'mlds').download_and_prepare()
    # Namespace 2: (non-existing)
    for d in tmp_path.iterdir():
      print(d)
    for d in (tmp_path / 'kaggle').iterdir():
      print(d)
    for d in (tmp_path / 'kaggle2').iterdir():
      print(d)
    for d in (tmp_path / 'mlds').iterdir():
      print(d)

    content = textwrap.dedent(f"""
        [kaggle]
        paths=[
            '{os.fspath(tmp_path / 'kaggle')}',
            '{os.fspath(tmp_path / 'kaggle2')}',
        ]
        info = "Kaggle datasets."
        [mlds]
        paths='{os.fspath(tmp_path / 'mlds')}'
        [other]
        paths='/tmp/path/to/non-existing-path'
        """)

    dummy_path = tmp_path / 'dummy-community-datasets.toml'
    dummy_path.write_text(content)
    config = config_lib.NamespaceRegistry(dummy_path)
    yield registry_lib.DatasetRegistry(config)


def test_register_builder(dummy_register):  # pylint: disable=redefined-outer-name
  builder = dummy_register.builder(naming.DatasetName('kaggle:ds0'))
  assert 'kaggle' in builder.data_path.parts

  # Same dataset name can be loaded from different namespace
  builder = dummy_register.builder(naming.DatasetName('mlds:ds0'))
  assert 'mlds' in builder.data_path.parts

  builder = dummy_register.builder(
      naming.DatasetName('mlds:ds0'),
      data_dir=None,  # data_dir can be passed only if None
      version='1.0.0',
  )
  assert 'mlds' in builder.data_path.parts

  with pytest.raises(ValueError, match='`data_dir` cannot be set for'):
    dummy_register.builder(
        naming.DatasetName('mlds:ds0'), data_dir='/path/to/data_dir'
    )

  with pytest.raises(
      registered.DatasetNotFoundError, match='Namespace .* not found.'
  ):
    dummy_register.builder(naming.DatasetName('non-existing-namespace:ds0'))

  with pytest.raises(registered.DatasetNotFoundError):
    dummy_register.builder(naming.DatasetName('other:ds0'))


def test_register_path_list_builders(dummy_register):  # pylint: disable=redefined-outer-name
  assert dummy_register.list_builders() == [
      'kaggle:ds0',
      'kaggle:ds1',
      'mlds:ds0',
  ]


def test_load_register_for_path_github():
  register = registry_lib._load_registry(
      namespace='huggingface',
      config=config_lib.NamespaceConfig(
          paths=['github://huggingface/datasets/tree/master/datasets']
      ),
  )
  assert isinstance(register, register_package.PackageRegister)
  assert register._path == gcs_utils.GCS_COMMUNITY_INDEX_PATH


def test_load_register_for_path_gcs():
  registers = registry_lib._load_registry(
      namespace='my_namespace',
      config=config_lib.NamespaceConfig(
          paths=['gs://my-bucket/datasets', 'gs://my-bucket2/datasets']
      ),
  )
  assert isinstance(registers, register_path.DataDirRegister)


def test_load_register_for_path_mixed():
  with pytest.raises(RuntimeError, match='Both a path containing .*'):
    registry_lib._load_registry(
        namespace='my_namespace',
        config=config_lib.NamespaceConfig(
            paths=[
                'github://huggingface/datasets/tree/master/datasets',
                'gs://my-bucket/datasets',
            ]
        ),
    )


def test_community_register():
  assert 'huggingface' in registry_lib.community_register().list_namespaces()


def _write_dummy_config(content: str, tmp_path: str) -> epath.Path:
  tmp_path = epath.Path(tmp_path)
  dummy_path = tmp_path / 'dummy-community-datasets.toml'
  dummy_path.write_text(content)
  return dummy_path


@mock.patch.object(registry_lib, '_registers_per_namespace')
def test_dataset_registry_list_builders(mock_registers_per_namespace, tmp_path):
  register1 = mock.create_autospec(register_path.DataDirRegister)
  register1.list_builders.return_value = ['a', 'b']
  register2 = mock.create_autospec(register_package.PackageRegister)
  register2.list_builders.return_value = ['c']
  mock_registers_per_namespace.return_value = {
      'ns1': [register1],
      'ns2': [register2],
  }
  dummy_path = _write_dummy_config(
      textwrap.dedent("""
    [ns1]
    paths = '/data/ds1'
    [ns2]
    paths = '/data/ds2'
  """),
      tmp_path,
  )
  config = config_lib.NamespaceRegistry(dummy_path)
  registry = registry_lib.DatasetRegistry(config)
  assert set(registry.list_namespaces()) == {'ns1', 'ns2'}
  assert set(registry.list_builders()) == {'a', 'b', 'c'}


@pytest.fixture()
def dummy_refs_and_registers():
  ref1 = naming.DatasetReference(dataset_name='ds1', namespace='ns1')
  ref2 = naming.DatasetReference(dataset_name='ds2', namespace='ns1')
  ref3 = naming.DatasetReference(dataset_name='ds3', namespace='ns2')
  register1 = mock.create_autospec(register_path.DataDirRegister)
  register1.list_dataset_references.return_value = [ref1, ref2]
  register2 = mock.create_autospec(register_package.PackageRegister)
  register2.list_dataset_references.return_value = [ref3]
  return {
      'dummy_references': [ref1, ref2, ref3],
      'dummy_registers': [register1, register2],
  }


@pytest.fixture()
def dummy_config(tmp_path):
  dummy_path = _write_dummy_config(
      textwrap.dedent("""
    [ns1]
    paths = '/data/ds1'
    [ns2]
    paths = '/data/ds2'
  """),
      tmp_path,
  )
  return config_lib.NamespaceRegistry(dummy_path)


@mock.patch.object(registry_lib, '_registers_per_namespace')
def test_list_dataset_references(
    mock_registers_per_namespace, dummy_refs_and_registers, dummy_config
):  # pylint: disable=redefined-outer-name
  register1, register2 = dummy_refs_and_registers['dummy_registers']
  mock_registers_per_namespace.return_value = {
      'ns1': [register1],
      'ns2': [register2],
  }
  registry = registry_lib.DatasetRegistry(dummy_config)
  assert (
      sorted(registry.list_dataset_references())
      == dummy_refs_and_registers['dummy_references']
  )


@mock.patch.object(registry_lib, '_registers_per_namespace')
def test_list_dataset_references_per_namespace(
    mock_registers_per_namespace, dummy_refs_and_registers, dummy_config
):  # pylint: disable=redefined-outer-name
  ref1, ref2, ref3 = dummy_refs_and_registers['dummy_references']
  register1, register2 = dummy_refs_and_registers['dummy_registers']
  mock_registers_per_namespace.return_value = {
      'ns1': [register1],
      'ns2': [register2],
  }
  registry = registry_lib.DatasetRegistry(dummy_config)
  assert sorted(
      registry.list_dataset_references_for_namespace(namespace='ns1')
  ) == [ref1, ref2]
  assert sorted(
      registry.list_dataset_references_for_namespace(namespace='ns2')
  ) == [ref3]

  with pytest.raises(ValueError, match='Namespace ns3 not found.'):
    list(registry.list_dataset_references_for_namespace(namespace='ns3'))


def test_add_namespace(dummy_register: registry_lib.DatasetRegistry):  # pylint: disable=redefined-outer-name
  mock_register = mock.create_autospec(register_path.DataDirRegister)
  dummy_register.add_namespace(
      'my_namespace',
      config=config_lib.NamespaceConfig(paths=[epath.Path('/data')]),
      registers=[mock_register],
  )
  assert 'my_namespace' in dummy_register.list_namespaces()
  assert 'my_namespace' in dummy_register.config_per_namespace
  assert 'my_namespace' in dummy_register.registers_per_namespace
  assert dummy_register.registers_per_namespace['my_namespace'] == [
      mock_register
  ]
  assert 'kaggle' in dummy_register.list_namespaces()


def test_add_namespace_duplicate(dummy_register: registry_lib.DatasetRegistry):  # pylint: disable=redefined-outer-name
  mock_register = mock.create_autospec(register_path.DataDirRegister)
  with pytest.raises(ValueError, match='Namespace kaggle already exists!.*'):
    dummy_register.add_namespace(
        'kaggle',
        config=config_lib.NamespaceConfig(paths=[epath.Path('/data')]),
        registers=[mock_register],
    )



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""opus dataset."""

import os

from absl import logging
from etils import epath
import tensorflow_datasets.public_api as tfds


class SubDataset:
  """Class to keep track of information on a sub-dataset of OPUS."""

  def __init__(self, name, description, homepage, url, languages):
    """Sub-dataset of OPUS.

    Args:
      name: `string`, a unique dataset identifier.
      description: `string`, a description of the dataset.
      homepage: `string`, homepage of the dataset.
      url: `string`, download url for the dataset.
      languages: `<list>(string)`, a list of supported languages.
    """
    self.name = name
    self.description = description
    self.homepage = homepage
    self.url = url

    sorted_languages = sorted(languages)
    language_pairs = []
    for idx, source in enumerate(sorted_languages):
      for target in sorted_languages[idx + 1 :]:
        language_pairs.append((source, target))

    self.language_pairs = language_pairs


DATASET_MAP = {  # pylint:disable=g-complex-comprehension
    ds.name: ds
    for ds in [
        # pylint:disable=line-too-long
        SubDataset(
            name="EMEA",
            description=(
                "A parallel corpus made out of PDF documents from the European"
                " Medicines Agency."
            ),
            homepage="http://opus.nlpl.eu/EMEA.php",
            url="http://opus.nlpl.eu/download.php?f=EMEA/v3/moses/",
            languages=[
                "bg",
                "cs",
                "da",
                "de",
                "el",
                "en",
                "es",
                "et",
                "fi",
                "fr",
                "hu",
                "it",
                "lt",
                "lv",
                "mt",
                "nl",
                "pl",
                "pt",
                "ro",
                "sk",
                "sl",
                "sv",
            ],
        ),
        SubDataset(
            name="JRC-Acquis",
            description=(
                "A collection of legislative text of the European Union and"
                " currently comprises selected texts written between the 1950s"
                " and now."
            ),
            homepage="http://opus.nlpl.eu/JRC-Acquis.php",
            url="http://opus.nlpl.eu/download.php?f=JRC-Acquis/",
            languages=[
                "bg",
                "cs",
                "da",
                "de",
                "el",
                "en",
                "es",
                "et",
                "fi",
                "fr",
                "hu",
                "it",
                "lt",
                "lv",
                "mt",
                "nl",
                "pl",
                "pt",
                "ro",
                "sk",
                "sl",
                "sv",
            ],
        ),
        SubDataset(
            name="Tanzil",
            description=(
                "A collection of Quran translations compiled by the Tanzil"
                " project."
            ),
            homepage="http://opus.nlpl.eu/Tanzil.php",
            url="http://opus.nlpl.eu/download.php?f=Tanzil/v1/moses/",
            languages=[
                "am",
                "ar",
                "az",
                "bg",
                "bn",
                "bs",
                "cs",
                "de",
                "dv",
                "en",
                "es",
                "fa",
                "fr",
                "ha",
                "hi",
                "id",
                "it",
                "ja",
                "ko",
                "ku",
                "ml",
                "ms",
                "nl",
                "no",
                "pl",
                "pt",
                "ro",
                "ru",
                "sd",
                "so",
                "sq",
                "sv",
                "sw",
                "ta",
                "tg",
                "th",
                "tr",
                "tt",
                "ug",
                "ur",
                "uz",
                "zh",
            ],
        ),
        SubDataset(
            name="GNOME",
            description=(
                "A parallel corpus of GNOME localization files. Source:"
                " https://l10n.gnome.org"
            ),
            homepage="http://opus.nlpl.eu/GNOME.php",
            url="http://opus.nlpl.eu/download.php?f=GNOME/v1/moses/",
            languages=[
                "af",
                "am",
                "an",
                "ang",
                "ar",
                "ar_TN",
                "ara",
                "as",
                "ast",
                "az",
                "az_IR",
                "bal",
                "be",
                "bem",
                "bg",
                "bg_BG",
                "bn",
                "bn_IN",
                "bo",
                "br",
                "brx",
                "bs",
                "ca",
                "cat",
                "crh",
                "cs",
                "csb",
                "cy",
                "da",
                "da_DK",
                "de",
                "de_CH",
                "dv",
                "dz",
                "el",
                "en",
                "en_AU",
                "en_CA",
                "en_GB",
                "en_NZ",
                "en_US",
                "en_ZA",
                "eo",
                "es",
                "es_AR",
                "es_CL",
                "es_CO",
                "es_CR",
                "es_DO",
                "es_EC",
                "es_ES",
                "es_GT",
                "es_HN",
                "es_MX",
                "es_NI",
                "es_PA",
                "es_PE",
                "es_PR",
                "es_SV",
                "es_UY",
                "es_VE",
                "et",
                "eu",
                "fa",
                "fa_IR",
                "fi",
                "fo",
                "foo",
                "fr",
                "fur",
                "fy",
                "ga",
                "gd",
                "gl",
                "gn",
                "gr",
                "gu",
                "gv",
                "ha",
                "he",
                "hi",
                "hi_IN",
                "hr",
                "hu",
                "hy",
                "ia",
                "id",
                "ig",
                "io",
                "is",
                "it",
                "it_IT",
                "ja",
                "jbo",
                "ka",
                "kg",
                "kk",
                "km",
                "kn",
                "ko",
                "kr",
                "ks",
                "ku",
                "ky",
                "la",
                "lg",
                "li",
                "lo",
                "lt",
                "lv",
                "mai",
                "mg",
                "mi",
                "mk",
                "ml",
                "mn",
                "mr",
                "ms",
                "ms_MY",
                "mt",
                "mus",
                "my",
                "nb",
                "nb_NO",
                "nds",
                "ne",
                "nhn",
                "nl",
                "nn",
                "nn_NO",
                "no",
                "no_nb",
                "nqo",
                "nr",
                "nso",
                "oc",
                "or",
                "os",
                "pa",
                "pl",
                "ps",
                "pt",
                "pt_BR",
                "pt_PT",
                "quz",
                "ro",
                "ru",
                "rw",
                "si",
                "sk",
                "sl",
                "so",
                "sq",
                "sr",
                "sr_ME",
                "st",
                "sv",
                "sw",
                "szl",
                "ta",
                "te",
                "tg",
                "tg_TJ",
                "th",
                "tk",
                "tl",
                "tl_PH",
                "tmp",
                "tr",
                "tr_TR",
                "ts",
                "tt",
                "ug",
                "uk",
                "ur",
                "ur_PK",
                "uz",
                "vi",
                "vi_VN",
                "wa",
                "xh",
                "yi",
                "yo",
                "zh_CN",
                "zh_HK",
                "zh_TW",
                "zu",
            ],
        ),
        SubDataset(
            name="KDE4",
            description="A parallel corpus of KDE4 localization files (v.2).",
            homepage="http://opus.nlpl.eu/KDE4.php",
            url="http://opus.nlpl.eu/download.php?f=KDE4/v2/moses/",
            languages=[
                "af",
                "ar",
                "as",
                "ast",
                "be",
                "bg",
                "bn",
                "bn_IN",
                "br",
                "ca",
                "crh",
                "cs",
                "csb",
                "cy",
                "da",
                "de",
                "el",
                "en",
                "en_GB",
                "eo",
                "es",
                "et",
                "eu",
                "fa",
                "fi",
                "fr",
                "fy",
                "ga",
                "gl",
                "gu",
                "ha",
                "he",
                "hi",
                "hne",
                "hr",
                "hsb",
                "hu",
                "hy",
                "id",
                "is",
                "it",
                "ja",
                "ka",
                "kk",
                "km",
                "kn",
                "ko",
                "ku",
                "lb",
                "lt",
                "lv",
                "mai",
                "mk",
                "ml",
                "mr",
                "ms",
                "mt",
                "nb",
                "nds",
                "ne",
                "nl",
                "nn",
                "nso",
                "oc",
                "or",
                "pa",
                "pl",
                "ps",
                "pt",
                "pt_BR",
                "ro",
                "ru",
                "rw",
                "se",
                "si",
                "sk",
                "sl",
                "sr",
                "sv",
                "ta",
                "te",
                "tg",
                "th",
                "tr",
                "uk",
                "uz",
                "vi",
                "wa",
                "xh",
                "zh_CN",
                "zh_HK",
                "zh_TW",
            ],
        ),
        SubDataset(
            name="PHP",
            description=(
                "A parallel corpus originally extracted from"
                " http://se.php.net/download-docs.php."
            ),
            homepage="http://opus.nlpl.eu/PHP.php",
            url="http://opus.nlpl.eu/download.php?f=PHP/v1/moses/",
            languages=[
                "cs",
                "de",
                "en",
                "es",
                "fi",
                "fr",
                "he",
                "hu",
                "it",
                "ja",
                "ko",
                "nl",
                "pl",
                "pt_BR",
                "ro",
                "ru",
                "sk",
                "sl",
                "sv",
                "tr",
                "tw",
                "zh",
                "zh_TW",
            ],
        ),
        SubDataset(
            name="Ubuntu",
            description=(
                "A parallel corpus of Ubuntu localization files. Source:"
                " https://translations.launchpad.net"
            ),
            homepage="http://opus.nlpl.eu/Ubuntu.php",
            url="http://opus.nlpl.eu/download.php?f=Ubuntu/v14.10/moses/",
            languages=[
                "ace",
                "af",
                "ak",
                "am",
                "an",
                "ang",
                "ar",
                "ar_SY",
                "ary",
                "as",
                "ast",
                "az",
                "ba",
                "bal",
                "be",
                "bem",
                "ber",
                "bg",
                "bho",
                "bn",
                "bn_IN",
                "bo",
                "br",
                "brx",
                "bs",
                "bua",
                "byn",
                "ca",
                "ce",
                "ceb",
                "chr",
                "ckb",
                "co",
                "crh",
                "cs",
                "csb",
                "cv",
                "cy",
                "da",
                "de",
                "de_AT",
                "de_DE",
                "dsb",
                "dv",
                "dz",
                "el",
                "en",
                "en_AU",
                "en_CA",
                "en_GB",
                "en_NZ",
                "en_US",
                "eo",
                "es",
                "es_AR",
                "es_CL",
                "es_CO",
                "es_CR",
                "es_DO",
                "es_EC",
                "es_ES",
                "es_GT",
                "es_HN",
                "es_MX",
                "es_NI",
                "es_PA",
                "es_PE",
                "es_PR",
                "es_SV",
                "es_UY",
                "es_VE",
                "et",
                "eu",
                "fa",
                "fa_AF",
                "ff",
                "fi",
                "fil",
                "fo",
                "fr",
                "fr_CA",
                "fr_FR",
                "frm",
                "frp",
                "fur",
                "fy",
                "ga",
                "gd",
                "gl",
                "gn",
                "grc",
                "gu",
                "guc",
                "gv",
                "ha",
                "haw",
                "he",
                "hi",
                "hil",
                "hne",
                "hr",
                "hsb",
                "ht",
                "hu",
                "hy",
                "ia",
                "id",
                "ig",
                "io",
                "is",
                "it",
                "iu",
                "ja",
                "jbo",
                "jv",
                "ka",
                "kab",
                "kg",
                "kk",
                "kl",
                "km",
                "kn",
                "ko",
                "kok",
                "ks",
                "ksh",
                "ku",
                "kw",
                "ky",
                "la",
                "lb",
                "lg",
                "li",
                "lij",
                "lld",
                "ln",
                "lo",
                "lt",
                "ltg",
                "lv",
                "mai",
                "mg",
                "mh",
                "mhr",
                "mi",
                "miq",
                "mk",
                "ml",
                "mn",
                "mo",
                "mr",
                "ms",
                "mt",
                "mus",
                "my",
                "nan",
                "nap",
                "nb",
                "nds",
                "ne",
                "nhn",
                "nl",
                "nl_NL",
                "nn",
                "no",
                "nso",
                "ny",
                "oc",
                "oj",
                "om",
                "or",
                "os",
                "pa",
                "pam",
                "pap",
                "pl",
                "pms",
                "pmy",
                "ps",
                "pt",
                "pt_BR",
                "pt_PT",
                "qu",
                "rm",
                "ro",
                "rom",
                "ru",
                "rw",
                "sa",
                "sc",
                "sco",
                "sd",
                "se",
                "shn",
                "shs",
                "si",
                "sk",
                "sl",
                "sm",
                "sml",
                "sn",
                "so",
                "son",
                "sq",
                "sr",
                "st",
                "sv",
                "sw",
                "syr",
                "szl",
                "ta",
                "ta_LK",
                "te",
                "tet",
                "tg",
                "th",
                "ti",
                "tk",
                "tl",
                "tlh",
                "tr",
                "trv",
                "ts",
                "tt",
                "ug",
                "uk",
                "ur",
                "uz",
                "ve",
                "vec",
                "vi",
                "wa",
                "wae",
                "wo",
                "xal",
                "xh",
                "yi",
                "yo",
                "zh",
                "zh_CN",
                "zh_HK",
                "zh_TW",
                "zu",
                "zza",
            ],
        ),
        SubDataset(
            name="OpenOffice",
            description=(
                "A collection of documents from http://www.openoffice.org/."
            ),
            homepage="http://opus.nlpl.eu/OpenOffice-v2.php",
            url="http://opus.nlpl.eu/download.php?f=OpenOffice/v2/moses/",
            languages=["de", "en", "es", "fr", "jp", "sv"],
        ),
        SubDataset(
            name="OpenSubtitles",
            description=(
                "A new collection of translated movie subtitles from"
                " http://www.opensubtitles.org/"
            ),
            homepage="http://opus.nlpl.eu/OpenSubtitles-v2018.php",
            url="http://opus.nlpl.eu/download.php?f=OpenSubtitles/v2018/moses/",
            languages=[
                "af",
                "ar",
                "bg",
                "bn",
                "br",
                "bs",
                "ca",
                "cs",
                "da",
                "de",
                "el",
                "en",
                "eo",
                "es",
                "et",
                "eu",
                "fa",
                "fi",
                "fr",
                "gl",
                "he",
                "hi",
                "hr",
                "hu",
                "hy",
                "id",
                "is",
                "it",
                "ja",
                "ka",
                "kk",
                "ko",
                "lt",
                "lv",
                "mk",
                "ml",
                "ms",
                "nl",
                "no",
                "pl",
                "pt",
                "pt_br",
                "ro",
                "ru",
                "si",
                "sk",
                "sl",
                "sq",
                "sr",
                "sv",
                "ta",
                "te",
                "th",
                "tl",
                "tr",
                "uk",
                "ur",
                "vi",
                "ze_en",
                "ze_zh",
                "zh_cn",
                "zh_tw",
            ],
        ),
    ]
}


class OpusConfig(tfds.core.BuilderConfig):
  """BuilderConfig for Opus."""

  def __init__(self, *, language_pair, subsets, **kwargs):
    """BuilderConfig for Opus.

    Args:
      language_pair: `(string, string)`, pair of languages used for translation.
        Should contain 2 letter coded strings (e.g. "de", "en")
      subsets: `<list>(string)`, list of the subdatasets to use.
      **kwargs: keyword arguments forwarded to super.
    """
    sorted_language_pair = sorted(language_pair)
    name = kwargs.get(
        "name",
        "%s-%s for %s"
        % (
            sorted_language_pair[0],
            sorted_language_pair[1],
            ", ".join(subsets),
        ),
    )

    description = name + " documents"

    super(OpusConfig, self).__init__(
        description=description, **dict(kwargs, name=name)
    )

    self.language_pair = sorted_language_pair
    self.subsets = subsets


class Builder(tfds.core.GeneratorBasedBuilder):
  """OPUS is a collection of translated texts from the web."""

  _KK_SUBSETS = [
      ("medical", ["EMEA"]),
      ("law", ["JRC-Acquis"]),
      ("koran", ["Tanzil"]),
      ("IT", ["GNOME", "KDE4", "PHP", "Ubuntu", "OpenOffice"]),
      ("subtitles", ["OpenSubtitles"]),
  ]
  """The following configurations reproduce the evaluation tasks from "Six

  Challenges for Neural Machine Translation" by Philipp Koehn and Rebecca
  Knowles (2017) https://www.aclweb.org/anthology/W17-3204.pdf
  """
  BUILDER_CONFIGS = [
      OpusConfig(  # pylint:disable=g-complex-comprehension
          version=tfds.core.Version("0.1.0"),
          language_pair=("de", "en"),
          subsets=subsets,
          name=name,
      )
      for name, subsets in _KK_SUBSETS
  ]

  @property
  def subsets(self):
    # Return only the datasets that exist for the language pair.
    source, target = self.builder_config.language_pair
    filtered_subsets = []
    for dataset in [DATASET_MAP[name] for name in self.builder_config.subsets]:
      if (source, target) in dataset.language_pairs:
        filtered_subsets.append(dataset)

    return filtered_subsets

  def _info(self):
    src, target = self.builder_config.language_pair
    return self.dataset_info_from_configs(
        features=tfds.features.Translation(
            languages=self.builder_config.language_pair
        ),
        supervised_keys=(src, target),
        homepage="http://opus.nlpl.eu/",
    )

  def _split_generators(self, dl_manager):
    source, target = self.builder_config.language_pair
    file_ext = "%s-%s" % (source, target)

    subsets = []
    for item in self.subsets:
      dl_dir = dl_manager.download_and_extract(
          os.path.join(item.url, "%s.txt.zip" % file_ext)
      )
      source_file = os.path.join(
          dl_dir, "%s.%s.%s" % (item.name, file_ext, source)
      )
      target_file = os.path.join(
          dl_dir, "%s.%s.%s" % (item.name, file_ext, target)
      )
      subsets.append({
          "name": item.name,
          "source_file": source_file,
          "target_file": target_file,
      })

    return [
        tfds.core.SplitGenerator(
            name=tfds.Split.TRAIN, gen_kwargs={"subsets": subsets}
        )
    ]

  def _generate_examples(self, subsets):
    source, target = self.builder_config.language_pair

    for item in subsets:
      logging.info("Generating examples from: %s", item["name"])
      source_file = item["source_file"]
      target_file = item["target_file"]

      gens = [_gen_line(source_file), _gen_line(target_file)]
      for idx, (source_sent, target_sent) in enumerate(zip(*gens)):
        result = {source: source_sent, target: target_sent}
        if all(result.values()):
          key = "%s/%d" % (item["name"], idx)
          yield key, result


def _gen_line(filename):
  """Returns sentences from an OPUS data file."""
  with epath.Path(filename).open() as f:
    for line in f:
      yield line



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Beans leaf dataset with images of diseased and health leaves."""

import os

import tensorflow_datasets.public_api as tfds

_TRAIN_URL = "https://storage.googleapis.com/ibeans/train.zip"
_VALIDATION_URL = "https://storage.googleapis.com/ibeans/validation.zip"
_TEST_URL = "https://storage.googleapis.com/ibeans/test.zip"

_IMAGE_SIZE = 500
_IMAGE_SHAPE = (_IMAGE_SIZE, _IMAGE_SIZE, 3)

_LABELS = ["angular_leaf_spot", "bean_rust", "healthy"]


class Builder(tfds.core.GeneratorBasedBuilder):
  """Beans plant leaf images dataset."""

  VERSION = tfds.core.Version("0.1.0")

  def _info(self):
    return self.dataset_info_from_configs(
        features=tfds.features.FeaturesDict({
            "image": tfds.features.Image(shape=_IMAGE_SHAPE),
            "label": tfds.features.ClassLabel(names=_LABELS),
        }),
        supervised_keys=("image", "label"),
        homepage="https://github.com/AI-Lab-Makerere/ibean/",
    )

  def _split_generators(self, dl_manager):
    train_path, val_path, test_path = dl_manager.download(
        [_TRAIN_URL, _VALIDATION_URL, _TEST_URL]
    )

    return [
        tfds.core.SplitGenerator(
            name=tfds.Split.TRAIN,
            gen_kwargs={"archive": dl_manager.iter_archive(train_path)},
        ),
        tfds.core.SplitGenerator(
            name=tfds.Split.VALIDATION,
            gen_kwargs={"archive": dl_manager.iter_archive(val_path)},
        ),
        tfds.core.SplitGenerator(
            name=tfds.Split.TEST,
            gen_kwargs={"archive": dl_manager.iter_archive(test_path)},
        ),
    ]

  def _generate_examples(self, archive):
    """Yields examples."""
    for fname, fobj in archive:
      if not fname.endswith(".jpg"):
        continue
      label = fname.split(os.path.sep)[-2]
      record = {
          "image": fobj,
          "label": label,
      }
      yield fname, record



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""bot_adversarial_dialogue dataset."""

import os
from typing import Any, Mapping

from etils import epath
import numpy as np
from tensorflow_datasets.core.utils import bool_utils
import tensorflow_datasets.public_api as tfds


_BOT_ADVERSARIAL_DIALOGUE_DATASETS_VERSION = "v0.2"
_HUMAN_NONADV_SAFETY_EVAL_TESTSET_VERSION = "v0.1"

# Class labels in "dialogue_datasets" and "human_nonadv_safety_eval" configs.
_LABELS = tfds.features.ClassLabel(names=["__ok__", "__notok__"])

# Features which are common to all configs.
_COMMON_FEATURES = {
    "id": tfds.features.Text(doc="The id of the sample."),
    "text": tfds.features.Text(doc="The utterance to classify."),
    "episode_done": np.bool_,
    "labels": _LABELS,
}

# Config-specific features.
_DIALOGUE_FEATURES = {
    "dialogue_id": np.float32,
    "round_id": np.float32,
    "speaker_to_eval": tfds.features.Text(
        doc="The speaker of the utterances labeled."
    ),
    "bot_persona": tfds.features.Sequence(
        tfds.features.Text(doc="The persona impersonated by the bot.")
    ),
}
_CONFIG_FEATURES = {
    "dialogue_datasets": tfds.features.FeaturesDict(
        {**_DIALOGUE_FEATURES, **_COMMON_FEATURES}
    ),
    "human_nonadv_safety_eval": tfds.features.FeaturesDict(_COMMON_FEATURES),
}


class Builder(tfds.core.GeneratorBasedBuilder):
  """DatasetBuilder for bot_adversarial_dialogue dataset."""

  VERSION = tfds.core.Version("1.0.0")
  RELEASE_NOTES = {
      "1.0.0": "Initial release.",
  }
  BUILDER_CONFIGS = [
      tfds.core.BuilderConfig(
          name="dialogue_datasets",
          description=(
              "The dialogue datasets, divided in train, validation and test"
              " splits."
          ),
      ),
      tfds.core.BuilderConfig(
          name="human_nonadv_safety_eval",
          description=(
              "An human safety evaluation set evaluated by crowdsourced workers"
              " for offensiveness. "
          ),
      ),
  ]
  DEFAULT_CONFIG_NAME = "dialogue_datasets"

  def _info(self) -> tfds.core.DatasetInfo:
    """Returns the dataset metadata."""
    return self.dataset_info_from_configs(
        features=_CONFIG_FEATURES[self.builder_config.name],
        supervised_keys=None,
        homepage="https://github.com/facebookresearch/ParlAI/tree/main/parlai/tasks/bot_adversarial_dialogue",
        license="https://github.com/facebookresearch/ParlAI/blob/main/LICENSE",
    )

  def _split_generators(self, dl_manager: tfds.download.DownloadManager):
    """Returns SplitGenerators."""

    bot_adversarial_dialogue_home = (
        "http://parl.ai/downloads/bot_adversarial_dialogue/"
    )

    if self.builder_config.name == "dialogue_datasets":
      path = dl_manager.download_and_extract(
          os.path.join(
              bot_adversarial_dialogue_home,
              f"dialogue_datasets_{_BOT_ADVERSARIAL_DIALOGUE_DATASETS_VERSION}.tar.gz",
          )
      )
      folder_name = "bot_adversarial_dialogue_datasets_with_persona"
      return {
          "train": self._generate_examples(
              path / f"{folder_name}/train.txt",
              split_name="train",
          ),
          "valid": self._generate_examples(
              path / f"{folder_name}/valid.txt",
              split_name="valid",
          ),
          "test": self._generate_examples(
              path / f"{folder_name}/test.txt",
              split_name="test",
          ),
      }

    else:
      path = dl_manager.download_and_extract(
          os.path.join(
              bot_adversarial_dialogue_home,
              f"human_nonadv_safety_eval_{_HUMAN_NONADV_SAFETY_EVAL_TESTSET_VERSION}.tar.gz",
          )
      )

      return {
          "test": self._generate_examples(
              path / "human_nonadv_safety_eval/test.txt",
              split_name="human_nonadv_safety_eval",
          ),
      }

  def _generate_examples(self, path, split_name=str):
    """Yields examples."""

    def _preprocess_row(row: str) -> str:
      """Preprocesses a dataset row using ParlAI format.

      This function is based on:
      https://github.com/facebookresearch/ParlAI/blob/9974b947fb2e801dc5608f495828532c2a714742/parlai/utils/misc.py#L639

      Args:
        row: An unprocessed row from the bot_adversarial_dialogue dataset.

      Returns:
        A processed row, in which special characters are properly formatted.
      """
      row = str(row)
      row = row.replace("\\t", "\t")
      row = row.replace("\\n", "\n")
      row = row.replace("__PIPE__", "|")
      return row

    def _get_row_features(row: str) -> Mapping[str, Any]:
      """Extracts dialogue features from a dataset row."""
      row_features = {}
      for field in row.split("\t"):
        key, value = field.split(":", maxsplit=1)
        row_features[key] = value
      return row_features

    # Indices to keep track of the dialogue turns as the conversation unfolds.
    previous_conversation_round = 0
    dialogue_id = 0

    with epath.Path(path).open() as f:
      for i, row in enumerate(f):
        example_id = f"{split_name}_{i}"
        cleaned_row = _preprocess_row(row)
        row_features = _get_row_features(cleaned_row)

        example = {
            "id": row_features.get("id", example_id),
            "labels": row_features["labels"],
            "episode_done": bool_utils.parse_bool(row_features["episode_done"]),
        }

        if self.builder_config.name == "dialogue_datasets":
          conversation_acts = row_features["text"].split("\n")
          conversation_len = len(conversation_acts)

          if conversation_len < previous_conversation_round:
            previous_conversation_round = 0
            dialogue_id += 1
          else:
            previous_conversation_round += 1

          # Remove the "your persona: ..." prefix from the bot persona
          # specifications.
          bot_persona = [
              str_.strip().split(": ", 1)[-1]
              for str_ in row_features["bot_persona"].strip().split("\n")
          ]

          example.update({
              "round_id": conversation_len - 1,
              "dialogue_id": dialogue_id,
              "text": conversation_acts[-1],
              "bot_persona": bot_persona,
              "speaker_to_eval": row_features["speaker_to_eval"],
          })

        else:
          example["text"] = row_features["text"]

        yield example_id, example



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""databricks_dolly dataset."""
import json

from etils import epath
import tensorflow_datasets.public_api as tfds


class Builder(tfds.core.GeneratorBasedBuilder):
  """DatasetBuilder for databricks_dolly dataset."""

  VERSION = tfds.core.Version('1.0.0')
  RELEASE_NOTES = {
      '1.0.0': 'Initial release.',
  }

  def _info(self) -> tfds.core.DatasetInfo:
    """Returns the dataset metadata."""
    return self.dataset_info_from_configs(
        features=tfds.features.FeaturesDict({
            'instruction': tfds.features.Text(),
            'context': tfds.features.Text(),
            'response': tfds.features.Text(),
            'category': tfds.features.Text(),
        }),
        homepage='https://github.com/databrickslabs/dolly',
        license='CC BY-SA 3.0',
    )

  def _split_generators(self, dl_manager: tfds.download.DownloadManager):
    path = dl_manager.download(
        {
            'train': 'https://github.com/databrickslabs/dolly/raw/master/data/databricks-dolly-15k.jsonl'
        }
    )
    return {
        'train': self._generate_examples(path['train']),
    }

  def _generate_examples(self, path):
    with epath.Path(path).open() as f:
      for idx, line in enumerate(f):
        if not line:
          continue
        content = json.loads(line)
        yield idx, content



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Caltech images dataset."""

import os

import numpy as np
from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
import tensorflow_datasets.public_api as tfds

_LABELS_FNAME = "image_classification/caltech101_labels.txt"
_URL = "https://data.caltech.edu/records/mzrjq-6wc02/files/caltech-101.zip?download=1"
_TRAIN_POINTS_PER_CLASS = 30


class Builder(tfds.core.GeneratorBasedBuilder):
  """Caltech-101."""

  VERSION = tfds.core.Version("3.0.2")
  RELEASE_NOTES = {
      "3.0.0": "New split API (https://tensorflow.org/datasets/splits)",
      "3.0.1": "Website URL update",
      "3.0.2": "Download URL update",
  }

  def _info(self):
    names_file = tfds.core.tfds_path(_LABELS_FNAME)
    return self.dataset_info_from_configs(
        features=tfds.features.FeaturesDict({
            "image": tfds.features.Image(),
            "label": tfds.features.ClassLabel(names_file=names_file),
            "image/file_name": tfds.features.Text(),  # E.g. 'image_0001.jpg'.
        }),
        supervised_keys=("image", "label"),
        homepage="https://doi.org/10.22002/D1.20086",
    )

  def _split_generators(self, dl_manager: tfds.download.DownloadManager):
    data_dir = dl_manager.download_and_extract(_URL)
    images_dir = dl_manager.extract(
        data_dir / "caltech-101" / "101_ObjectCategories.tar.gz"
    )

    return {
        "train": self._generate_examples(images_dir, is_train_split=True),
        "test": self._generate_examples(images_dir, is_train_split=False),
    }

  def _generate_examples(self, images_dir_path, is_train_split):
    """Generates images and labels given the image directory path.

    As is usual for this dataset, 30 random examples from each class are added
    to the train split, and the remainder are added to the test split.

    Args:
      images_dir_path: path to the directory where the images are stored.
      is_train_split: bool, if true, generates the train split, else generates
        the test split.

    Yields:
      The image path, and its corresponding label and filename.

    Raises:
      ValueError: If too few points are present to create the train set for any
        class.
    """
    # Sets random seed so the random partitioning of files is the same when
    # called for the train and test splits.
    numpy_original_state = np.random.get_state()
    np.random.seed(1234)

    parent_dir = tf.io.gfile.listdir(images_dir_path)[0]
    walk_dir = os.path.join(images_dir_path, parent_dir)
    dirs = tf.io.gfile.listdir(walk_dir)

    for d in dirs:
      # Each directory contains all the images from a single class.
      if tf.io.gfile.isdir(os.path.join(walk_dir, d)):
        for full_path, _, fnames in tf.io.gfile.walk(os.path.join(walk_dir, d)):
          # _TRAIN_POINTS_PER_CLASS datapoints are sampled for the train split,
          # the others constitute the test split.
          if _TRAIN_POINTS_PER_CLASS > len(fnames):
            raise ValueError(
                "Fewer than {} ({}) points in class {}".format(
                    _TRAIN_POINTS_PER_CLASS, len(fnames), d
                )
            )
          train_fnames = np.random.choice(
              fnames, _TRAIN_POINTS_PER_CLASS, replace=False
          )
          test_fnames = set(fnames).difference(train_fnames)
          fnames_to_emit = train_fnames if is_train_split else test_fnames

          for image_file in fnames_to_emit:
            if image_file.endswith(".jpg"):
              image_path = os.path.join(full_path, image_file)
              record = {
                  "image": image_path,
                  "label": d.lower(),
                  "image/file_name": image_file,
              }
              yield "%s/%s" % (d, image_file), record
    # Resets the seeds to their previous states.
    np.random.set_state(numpy_original_state)



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# pylint: skip-file

# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: smart_control_normalization.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n!smart_control_normalization.proto\x12#smart_buildings.smart_control.proto\x1a\x1fgoogle/protobuf/timestamp.proto"\x90\x02\n\x16\x43ontinuousVariableInfo\x12\n\n\x02id\x18\x01'
    b' \x01(\t\x12\x30\n\x0csample_start\x18\x02'
    b' \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12.\n\nsample_end\x18\x03'
    b' \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x13\n\x0bsample_size\x18\x04'
    b' \x01(\x05\x12\x17\n\x0fsample_variance\x18\x05'
    b' \x01(\x02\x12\x13\n\x0bsample_mean\x18\x06'
    b' \x01(\x02\x12\x15\n\rsample_median\x18\x07'
    b' \x01(\x02\x12\x16\n\x0esample_maximum\x18\x08'
    b' \x01(\x02\x12\x16\n\x0esample_minimum\x18\t \x01(\x02\x62\x06proto3'
)

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(
    DESCRIPTOR, 'smart_control_normalization_pb2', globals()
)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _CONTINUOUSVARIABLEINFO._serialized_start = 108
  _CONTINUOUSVARIABLEINFO._serialized_end = 380
# @@protoc_insertion_point(module_scope)



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Adapters for file formats."""

from __future__ import annotations

import abc
from collections.abc import Iterable, Iterator
import concurrent.futures
import enum
import itertools
import os
import re
from typing import Any, ClassVar, Type, TypeVar

from etils import epy
from tensorflow_datasets.core.utils.lazy_imports_utils import apache_beam as beam
from tensorflow_datasets.core.utils.lazy_imports_utils import array_record_data_source
from tensorflow_datasets.core.utils.lazy_imports_utils import array_record_module
from tensorflow_datasets.core.utils.lazy_imports_utils import parquet as pq
from tensorflow_datasets.core.utils.lazy_imports_utils import pyarrow as pa
from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf

with epy.lazy_imports():
  # pylint: disable=g-import-not-at-top
  from etils import epath
  from tensorflow_datasets.core import naming
  from tensorflow_datasets.core.utils import file_utils
  from tensorflow_datasets.core.utils import type_utils

  # pylint: enable=g-import-not-at-top

ExamplePositions = list[Any]
T = TypeVar('T')


class FileFormat(enum.Enum):
  """Format of the record files.

  The values of the enumeration are used as filename endings/suffix.
  """

  TFRECORD = 'tfrecord'
  RIEGELI = 'riegeli'
  ARRAY_RECORD = 'array_record'
  PARQUET = 'parquet'

  @property
  def file_suffix(self) -> str:
    return ADAPTER_FOR_FORMAT[self].FILE_SUFFIX

  def deserialize(self, raw_example: bytes) -> Any:
    """Deserializes bytes into an object, but does not decode features."""
    return ADAPTER_FOR_FORMAT[self].deserialize(raw_example)

  @classmethod
  def with_random_access(cls) -> set[FileFormat]:
    """File formats with random access."""
    return {
        file_format
        for file_format, adapter in ADAPTER_FOR_FORMAT.items()
        if adapter.SUPPORTS_RANDOM_ACCESS
    }

  @classmethod
  def with_tf_data(cls) -> set[FileFormat]:
    """File formats with tf.data support."""
    return {
        file_format
        for file_format, adapter in ADAPTER_FOR_FORMAT.items()
        if adapter.SUPPORTS_TF_DATA
    }

  @classmethod
  def with_suffix_before_shard_spec(cls) -> set[FileFormat]:
    """File formats with suffix before shard spec."""
    return {
        file_format
        for file_format, adapter in ADAPTER_FOR_FORMAT.items()
        if adapter.SUFFIX_BEFORE_SHARD_SPEC
    }

  @classmethod
  def with_suffix_after_shard_spec(cls) -> set[FileFormat]:
    """File formats with suffix after shard spec."""
    return {
        file_format
        for file_format, adapter in ADAPTER_FOR_FORMAT.items()
        if not adapter.SUFFIX_BEFORE_SHARD_SPEC
    }

  @classmethod
  def from_value(cls, file_format: str | FileFormat) -> FileFormat:
    try:
      return cls(file_format)
    except ValueError as e:
      all_values = [f.value for f in cls]
      raise ValueError(
          f'{file_format} is not a valid FileFormat! '
          f'Valid file formats: {all_values}'
      ) from e


DEFAULT_FILE_FORMAT = FileFormat.TFRECORD


class FileAdapter(abc.ABC):
  """Interface for Adapter objects which read and write examples in a format."""

  FILE_SUFFIX: ClassVar[str]

  # Whether the file format suffix should go before the shard spec.
  # For example, `dataset-train.tfrecord-00000-of-00001` if `True`,
  # otherwise `dataset-train-00000-of-00001.tfrecord`.
  SUFFIX_BEFORE_SHARD_SPEC: ClassVar[bool] = True

  SUPPORTS_RANDOM_ACCESS: ClassVar[bool]
  SUPPORTS_TF_DATA: ClassVar[bool]
  BUFFER_SIZE = 8 << 20  # 8 MiB per file.

  @classmethod
  @abc.abstractmethod
  def make_tf_data(
      cls,
      filename: epath.PathLike,
      buffer_size: int | None = None,
  ) -> tf.data.Dataset:
    """Returns TensorFlow Dataset comprising given record file."""
    raise NotImplementedError()

  @classmethod
  @abc.abstractmethod
  def write_examples(
      cls,
      path: epath.PathLike,
      iterator: Iterable[type_utils.KeySerializedExample],
  ) -> ExamplePositions | None:
    """Write examples from given iterator in given path.

    Args:
      path: Path where to write the examples.
      iterator: Iterable of examples.

    Returns:
      List of record positions for each record in the given iterator. In case of
      TFRecords, does not return anything.
    """
    raise NotImplementedError()

  @classmethod
  def deserialize(cls, raw_example: bytes) -> Any:
    """Returns the deserialized example, but does not decode features.

    If custom serialization is used, override this method in the file adapter.

    Args:
      raw_example: the bytes read from the source that should be deserialized.
    """
    return tf.train.Example.FromString(raw_example)

  @classmethod
  def beam_sink(
      cls,
      filename_template: naming.ShardedFileTemplate,
      num_shards: int | None = None,
  ) -> beam.PTransform:
    """Returns a Beam sink for writing examples in the given file format."""
    raise NotImplementedError()

  @classmethod
  def num_examples(cls, filename: epath.PathLike) -> int:
    """Returns the number of examples in the given file."""
    n = 0
    for _ in cls.make_tf_data(filename):
      n += 1
    return n

  @classmethod
  def shard_lengths_and_sizes(
      cls,
      filename_template: naming.ShardedFileTemplate,
      num_shards: int | None = None,
  ) -> list[tuple[int, int]]:
    """Returns the number of examples in each shard."""
    if num_shards is not None:
      shards = filename_template.sharded_filepaths(num_shards=num_shards)
    else:
      shards = filename_template.data_dir.glob(filename_template.glob_pattern())
    shards = sorted([os.fspath(s) for s in shards])

    def _get_length_and_size(shard: tuple[int, str]) -> tuple[int, int, int]:
      index, shard = shard
      length = cls.num_examples(shard)
      size = epath.Path(shard).stat().length
      return index, length, size

    with concurrent.futures.ThreadPoolExecutor(max_workers=16) as executor:
      results = executor.map(_get_length_and_size, enumerate(shards))
    # Sort results by the index and remove the index from the tuple.
    sorted_results = sorted(results, key=lambda x: x[0])
    return [(length, size) for _, length, size in sorted_results]


class TfRecordFileAdapter(FileAdapter):
  """File adapter for TFRecord file format."""

  FILE_SUFFIX = 'tfrecord'
  SUPPORTS_RANDOM_ACCESS = False
  SUPPORTS_TF_DATA = True

  @classmethod
  def make_tf_data(
      cls,
      filename: epath.PathLike,
      buffer_size: int | None = None,
  ) -> tf.data.Dataset:
    """Returns TensorFlow Dataset comprising given record file."""
    buffer_size = buffer_size or cls.BUFFER_SIZE
    return tf.data.TFRecordDataset(filename, buffer_size=buffer_size)

  @classmethod
  def write_examples(
      cls,
      path: epath.PathLike,
      iterator: Iterable[type_utils.KeySerializedExample],
  ) -> ExamplePositions | None:
    """Write examples from given iterator in given path.

    Args:
      path: Path where to write the examples.
      iterator: Iterable of examples.

    Returns:
      None
    """
    with tf.io.TFRecordWriter(os.fspath(path)) as writer:
      for _, serialized_example in iterator:
        writer.write(serialized_example)
      writer.flush()

  @classmethod
  def beam_sink(
      cls,
      filename_template: naming.ShardedFileTemplate,
      num_shards: int | None = None,
  ) -> beam.PTransform:
    """Returns a Beam sink for writing examples in the given file format."""
    file_path_prefix = filename_template.sharded_filepaths_pattern(
        # num_shards cannot be both in the path and passed as an argument, so
        # make sure it's not in the path.
        num_shards=None,
        use_at_notation=True,
    ).removesuffix('@*')
    return beam.io.WriteToTFRecord(
        file_path_prefix=file_path_prefix, num_shards=num_shards
    )


class RiegeliFileAdapter(FileAdapter):
  """File adapter for Riegeli file format."""

  FILE_SUFFIX = 'riegeli'
  SUPPORTS_RANDOM_ACCESS = False
  SUPPORTS_TF_DATA = True

  @classmethod
  def make_tf_data(
      cls,
      filename: epath.PathLike,
      buffer_size: int | None = None,
  ) -> tf.data.Dataset:
    buffer_size = buffer_size or cls.BUFFER_SIZE
    from riegeli.tensorflow.ops import riegeli_dataset_ops as riegeli_tf  # pylint: disable=g-import-not-at-top  # pytype: disable=import-error

    return riegeli_tf.RiegeliDataset(filename, buffer_size=buffer_size)

  @classmethod
  def write_examples(
      cls,
      path: epath.PathLike,
      iterator: Iterable[type_utils.KeySerializedExample],
  ) -> ExamplePositions | None:
    """Write examples from given iterator in given path.

    Args:
      path: Path where to write the examples.
      iterator: Iterable of examples.

    Returns:
      List of record positions for each record in the given iterator.
    """
    positions = []
    import riegeli  # pylint: disable=g-import-not-at-top

    with tf.io.gfile.GFile(os.fspath(path), 'wb') as f:
      with riegeli.RecordWriter(f, options='transpose') as writer:
        for _, record in iterator:
          writer.write_record(record)
          positions.append(writer.last_pos)
    return positions


class ArrayRecordFileAdapter(FileAdapter):
  """File adapter for ArrayRecord file format."""

  FILE_SUFFIX = 'array_record'
  SUPPORTS_RANDOM_ACCESS = True
  SUPPORTS_TF_DATA = False

  @classmethod
  def make_tf_data(
      cls,
      filename: epath.PathLike,
      buffer_size: int | None = None,
  ) -> tf.data.Dataset:
    """Returns TensorFlow Dataset comprising given array record file."""
    raise NotImplementedError(
        '`.as_dataset()` not implemented for ArrayRecord files. Please, use'
        ' `.as_data_source()`.'
    )

  @classmethod
  def write_examples(
      cls,
      path: epath.PathLike,
      iterator: Iterable[type_utils.KeySerializedExample],
  ) -> ExamplePositions | None:
    """Write examples from given iterator in given path.

    Args:
      path: Path where to write the examples.
      iterator: Iterable of examples.

    Returns:
      None
    """
    writer = array_record_module.ArrayRecordWriter(
        os.fspath(path), 'group_size:1'
    )
    for _, serialized_example in iterator:
      writer.write(serialized_example)
    writer.close()

  @classmethod
  def num_examples(cls, filename: epath.PathLike) -> int:
    """Returns the number of examples in the given file."""
    data_source = array_record_data_source.ArrayRecordDataSource(
        paths=[os.fspath(filename)]
    )
    return len(data_source)


class ParquetFileAdapter(FileAdapter):
  """File adapter for the [Parquet](https://parquet.apache.org) file format.

  This FileAdapter requires `pyarrow` as a dependency and builds upon
  `pyarrow.parquet`.

  At the moment, the Parquet adapter doesn't leverage Parquet's columnar
  features and behaves like any other adapter. Instead of saving the features in
  the columns, we use one single `data` column where we store the serialized
  tf.Example proto.

  TODO(b/317277518): Let Parquet handle the serialization/deserialization.
  """

  FILE_SUFFIX = 'parquet'
  SUPPORTS_RANDOM_ACCESS = True
  SUPPORTS_TF_DATA = True
  _PARQUET_FIELD = 'data'
  _BATCH_SIZE = 100

  @classmethod
  def _schema(cls) -> pa.Schema:
    """Returns the Parquet schema as a one-column `data` binary field."""
    return pa.schema([pa.field(cls._PARQUET_FIELD, pa.binary())])

  @classmethod
  def make_tf_data(
      cls,
      filename: epath.PathLike,
      buffer_size: int | None = None,
  ) -> tf.data.Dataset:
    """Reads a Parquet file as a tf.data.Dataset.

    Args:
      filename: Path to the Parquet file.
      buffer_size: Unused buffer size.

    Returns:
      A tf.data.Dataset with the serialized examples.
    """
    del buffer_size  # unused

    def get_data(py_filename: bytes) -> Iterator[tf.Tensor]:
      table = pq.read_table(py_filename.decode(), schema=cls._schema())
      for batch in table.to_batches():
        for example in batch.to_pylist():
          yield tf.constant(example[cls._PARQUET_FIELD])

    return tf.data.Dataset.from_generator(
        get_data,
        args=(filename,),
        output_signature=tf.TensorSpec(shape=(), dtype=tf.string),
    )

  @classmethod
  def write_examples(
      cls,
      path: epath.PathLike,
      iterator: Iterable[type_utils.KeySerializedExample],
  ) -> None:
    """Writes the serialized tf.Example proto in a binary field named `data`.

    Args:
      path: Path to the Parquet file.
      iterator: Iterable of serialized examples.
    """
    with pq.ParquetWriter(path, schema=cls._schema()) as writer:
      for examples in _batched(iterator, cls._BATCH_SIZE):
        examples = [{cls._PARQUET_FIELD: example} for _, example in examples]
        batch = pa.RecordBatch.from_pylist(examples)
        writer.write_batch(batch)
    return None


def _to_bytes(key: type_utils.Key) -> bytes:
  """Convert the key to bytes."""
  if isinstance(key, int):
    return key.to_bytes(128, byteorder='big')  # Use 128 as this match md5
  elif isinstance(key, bytes):
    return key
  elif isinstance(key, str):
    return key.encode('utf-8')
  else:
    raise TypeError(f'Invalid key type: {type(key)}')


# Create a mapping from FileFormat -> FileAdapter.
ADAPTER_FOR_FORMAT: dict[FileFormat, Type[FileAdapter]] = {
    FileFormat.ARRAY_RECORD: ArrayRecordFileAdapter,
    FileFormat.PARQUET: ParquetFileAdapter,
    FileFormat.RIEGELI: RiegeliFileAdapter,
    FileFormat.TFRECORD: TfRecordFileAdapter,
}

_FILE_SUFFIX_TO_FORMAT = {
    adapter.FILE_SUFFIX: file_format
    for file_format, adapter in ADAPTER_FOR_FORMAT.items()
}


def file_format_from_suffix(file_suffix: str) -> FileFormat:
  """Returns the file format associated with the file extension (`tfrecord`)."""
  if file_suffix not in _FILE_SUFFIX_TO_FORMAT:
    raise ValueError(
        'Unrecognized file extension: Should be one of '
        f'{list(_FILE_SUFFIX_TO_FORMAT.values())}'
    )
  return _FILE_SUFFIX_TO_FORMAT[file_suffix]


def is_example_file(filename: str) -> bool:
  """Whether the given filename is a record file."""
  return any(
      f'.{adapter.FILE_SUFFIX}' in filename
      for adapter in ADAPTER_FOR_FORMAT.values()
  )


def _batched(iterator: Iterator[T] | Iterable[T], n: int) -> Iterator[list[T]]:
  """Batches the result of an iterator into lists of length n.

  This function is built-in the standard library from 3.12 (source:
  https://docs.python.org/3/library/itertools.html#itertools.batched). However,
  TFDS supports older versions of Python.

  Args:
    iterator: The iterator to batch.
    n: The maximal length of each batch.

  Yields:
    The next list of n elements.
  """
  i = 0
  while True:
    batch = list(itertools.islice(iterator, i, i + n))
    if not batch:
      return
    yield batch
    i += n


def convert_path_to_file_format(
    path: epath.PathLike, file_format: FileFormat
) -> epath.Path:
  """Returns the path to a specific shard for a different file format.

  TFDS can store the file format in the filename as a suffix or as an infix. For
  example:

  - `dataset-train.<FILE_FORMAT>-00000-of-00001`, a so-called infix format
    because the file format comes before the shard spec.
  - `dataset-train-00000-of-00001.<FILE_FORMAT>`, a so-called suffix format
    because the file format comes after the shard spec.

  Args:
    path: The path of a specific to convert. Can be the path for different file
      formats.
    file_format: The file format to which the shard path should be converted.
  """
  path = epath.Path(path)
  file_name: str = path.name
  if file_format.file_suffix in file_name:
    # Already has the right file format in the file name.
    return path

  infix_formats = FileFormat.with_suffix_before_shard_spec()
  suffix_formats = FileFormat.with_suffix_after_shard_spec()

  # Remove any existing file format from the file name.
  infix_format_concat = '|'.join(f.file_suffix for f in infix_formats)
  file_name = re.sub(rf'(\.({infix_format_concat}))', '', file_name)

  suffix_formats_concat = '|'.join(f.file_suffix for f in suffix_formats)
  file_name = re.sub(rf'(\.({suffix_formats_concat}))$', '', file_name)

  # Add back the proper file format.
  if file_format in suffix_formats:
    file_name = f'{file_name}.{file_format.file_suffix}'
  else:
    file_name = re.sub(
        r'-(\d+)-of-(\d+)',
        rf'.{file_format.file_suffix}-\1-of-\2',
        file_name,
    )
  return path.parent / file_name



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""This module contains the reader config."""

from __future__ import annotations

from collections.abc import Sequence
import dataclasses
from typing import Callable, cast

from tensorflow_datasets.core import file_adapters
from tensorflow_datasets.core.utils import shard_utils
from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf

InterleaveSortFn = Callable[
    [Sequence[shard_utils.FileInstruction]],
    Sequence[shard_utils.FileInstruction],
]


class _MISSING(str):
  pass


MISSING = _MISSING('missing')


@dataclasses.dataclass(eq=False)
class ReadConfig:
  # pyformat: disable
  """Configures input reading pipeline.

  Attributes:
    options: `tf.data.Options()`, dataset options to use. Note that when
      `shuffle_files` is True and no seed is defined, deterministic will be set
      to False internally, unless it is defined here.
    try_autocache: If True (default) and the dataset satisfy the right
      conditions (dataset small enough, files not shuffled,...) the dataset will
      be cached during the first iteration (through `ds = ds.cache()`).
    repeat_filenames: If True, repeat the filenames iterator. This will result
      in an infinite dataset. Repeat is called after the shuffle of the
      filenames.
    add_tfds_id: If True, examples `dict` in `tf.data.Dataset` will have an
      additional key `'tfds_id': tf.Tensor(shape=(), dtype=tf.string)`
      containing the example unique identifier (e.g.
      'train.tfrecord-000045-of-001024__123').
       Note: IDs might changes in future version of TFDS.
    shuffle_seed: `tf.int64`, seed forwarded to `tf.data.Dataset.shuffle` during
      file shuffling (which happens when `tfds.load(..., shuffle_files=True)`).
    shuffle_reshuffle_each_iteration: `bool`, forwarded to
      `tf.data.Dataset.shuffle` during file shuffling (which happens when
      `tfds.load(..., shuffle_files=True)`).
    interleave_cycle_length: `int`, forwarded to `tf.data.Dataset.interleave`.
    interleave_block_length: `int`, forwarded to `tf.data.Dataset.interleave`.
    input_context: `tf.distribute.InputContext`, if set, each worker will read a
      different set of file. For more info, see the
      [distribute_datasets_from_function
      documentation](https://www.tensorflow.org/api_docs/python/tf/distribute/Strategy#experimental_distribute_datasets_from_function).
      Note:  * Each workers will always read the same subset of files.
        `shuffle_files` only shuffle files within each worker. * If
        `info.splits[split].num_shards < input_context.num_input_pipelines`, an
        error will be raised, as some workers would be empty.
    experimental_interleave_sort_fn: Function with signature `List[FileDict] ->
      List[FileDict]`, which takes the list of `dict(file: str, take: int, skip:
      int)` and returns the modified version to read. This can be used to
      sort/shuffle the shards to read in a custom order, instead of relying on
      `shuffle_files=True`.
    skip_prefetch: If False (default), add a `ds.prefetch()` op at the end.
      Might be set for performance optimization in some cases (e.g. if you're
      already calling `ds.prefetch()` at the end of your pipeline)
    num_parallel_calls_for_decode: The number of parallel calls for decoding
      record. By default using tf.data's AUTOTUNE.
    num_parallel_calls_for_interleave_files: The number of parallel calls for
      interleaving files. By default using tf.data's AUTOTUNE.
    enable_ordering_guard: When True (default), an exception is raised if
      shuffling or interleaving are used on an ordered dataset.
    assert_cardinality: When True (default), an exception is raised if at the
      end of an Epoch the number of read examples does not match the expected
      number from dataset metadata. A power user would typically want to set
      False if input files have been tempered with and they don't mind missing
      records or have too many of them.
    override_buffer_size: number of bytes to pass to file readers for buffering.
    file_format: if the dataset is stored in multiple file formats, then this
      argument can be used to specify the file format to load. If not specified,
      the default file format is used.
  """
  # pyformat: enable

  # General tf.data.Dataset parametters
  options: tf.data.Options | None = None
  try_autocache: bool = True
  repeat_filenames: bool = False
  add_tfds_id: bool = False
  # tf.data.Dataset.shuffle parameters
  shuffle_seed: int | None = None
  shuffle_reshuffle_each_iteration: bool | None = None
  # Interleave parameters
  # Ideally, we should switch interleave values to None to dynamically set
  # those value depending on the user system. However, this would make the
  # generation order non-deterministic accross machines.
  interleave_cycle_length: int | None | _MISSING = MISSING
  interleave_block_length: int | None = 16
  input_context: tf.distribute.InputContext | None = None
  experimental_interleave_sort_fn: InterleaveSortFn | None = None
  skip_prefetch: bool = False
  num_parallel_calls_for_decode: int | None = None
  # Cast to an `int`. `__post_init__` will ensure the type invariant.
  num_parallel_calls_for_interleave_files: int | None = cast(int, MISSING)
  enable_ordering_guard: bool = True
  assert_cardinality: bool = True
  override_buffer_size: int | None = None
  file_format: str | file_adapters.FileFormat | None = None

  def __post_init__(self):
    self.options = self.options or tf.data.Options()
    if self.num_parallel_calls_for_decode is None:
      self.num_parallel_calls_for_decode = tf.data.AUTOTUNE
    if self.num_parallel_calls_for_interleave_files == MISSING:
      self.num_parallel_calls_for_interleave_files = tf.data.AUTOTUNE
    if isinstance(self.file_format, str):
      self.file_format = file_adapters.FileFormat.from_value(self.file_format)

  def replace(self, **kwargs) -> ReadConfig:
    return dataclasses.replace(self, **kwargs)



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Opinion Abstracts Dataset."""

from __future__ import annotations

import json
import os
from typing import Any, Dict, Iterator, List, Optional, Text, Tuple

import numpy as np
from tensorflow_datasets.core.utils.lazy_imports_utils import tensorflow as tf
import tensorflow_datasets.public_api as tfds

_URL = "https://web.eecs.umich.edu/~wangluxy/datasets/opinion_abstracts.zip"


class OpinionAbstractsConfig(tfds.core.BuilderConfig):
  """BuilderConfig for OpinionAbstracts."""

  def __init__(
      self,
      *,
      filename: Optional[Text] = None,
      name_key: Optional[Text] = None,
      id_key: Optional[Text] = None,
      opinions_key: Optional[Text] = None,
      summary_key: Optional[Text] = None,
      **kwargs,
  ):
    """BuilderConfig for OpinionAbstracts."""
    super(OpinionAbstractsConfig, self).__init__(
        version=tfds.core.Version("1.0.0"), **kwargs
    )
    self.filename = filename
    self.name_key = name_key
    self.id_key = id_key
    self.opinions_key = opinions_key
    self.summary_key = summary_key


class Builder(tfds.core.GeneratorBasedBuilder):
  """OpinionAbstracts Dataset Builder."""

  VERSION = tfds.core.Version("1.0.0")
  BUILDER_CONFIGS = [
      OpinionAbstractsConfig(
          name="rotten_tomatoes",
          filename="rottentomatoes.json",
          name_key="_movie_name",
          id_key="_movie_id",
          opinions_key="_critics",
          summary_key="_critic_consensus",
          description="Professional critics and consensus of 3,731 movies.",
      ),
      OpinionAbstractsConfig(
          name="idebate",
          filename="idebate.json",
          name_key="_debate_name",
          id_key="_claim_id",
          opinions_key="_argument_sentences",
          summary_key="_claim",
          description="2,259 claims for 676 debates.",
      ),
  ]

  def _info(self) -> tfds.core.DatasetInfo:
    config = self.builder_config
    return self.dataset_info_from_configs(
        features=tfds.features.FeaturesDict({
            config.name_key: np.str_,
            config.id_key: np.str_,
            config.summary_key: np.str_,
            config.opinions_key: tfds.features.Sequence(
                tfds.features.FeaturesDict({"key": np.str_, "value": np.str_})
            ),
        }),
        supervised_keys=(config.opinions_key, config.summary_key),
        homepage="https://web.eecs.umich.edu/~wangluxy/data.html",
    )

  def _split_generators(
      self, dl_manager: tfds.download.DownloadManager
  ) -> List[tfds.core.SplitGenerator]:
    """Returns SplitGenerators."""
    dl_path = dl_manager.download_and_extract(_URL)
    path = os.path.join(
        dl_path, "opinion_abstracts", self.builder_config.filename
    )
    return [
        tfds.core.SplitGenerator(
            name=tfds.Split.TRAIN,
            gen_kwargs={"path": path},
        ),
    ]

  def _generate_examples(
      self, path: Optional[Text] = None
  ) -> Iterator[Tuple[Text, Dict[Text, Any]]]:
    """Yields examples."""
    with tf.io.gfile.GFile(path, "rb") as f:
      for example in json.load(f):
        config = self.builder_config
        opinions = example[config.opinions_key].items()
        opinions = [{"key": k, "value": v} for k, v in opinions]
        features = {config.opinions_key: opinions}
        for k in [config.name_key, config.id_key, config.summary_key]:
          features[k] = example[k]
        yield example[config.id_key], features



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""natural_instructions dataset."""

from tensorflow_datasets import testing
from tensorflow_datasets.datasets.natural_instructions import natural_instructions_dataset_builder


class NaturalInstructionsTest(testing.DatasetBuilderTestCase):
  """Tests for natural_instructions dataset."""

  DATASET_CLASS = natural_instructions_dataset_builder.Builder
  SPLITS = {
      'train': 3,  # Number of fake train example
  }

  DL_EXTRACT_RESULT = {'train': 'natural_instructions_train.json'}

  # If you are calling `download/download_and_extract` with a dict, like:
  #   dl_manager.download({'some_key': 'http://a.org/out.txt', ...})
  # then the tests needs to provide the fake output paths relative to the
  # fake data directory
  # DL_EXTRACT_RESULT = {'some_key': 'output_file1.txt', ...}


if __name__ == '__main__':
  testing.test_main()



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Test QM9 dataset conversion to tensorflow_datasets format."""

from tensorflow_datasets import testing
from tensorflow_datasets.datasets.qm9 import qm9_dataset_builder


class Qm9Test(testing.DatasetBuilderTestCase):
  """Tests for QM9 dataset. See superclass for details."""

  # We patch properties of the testdata here.
  qm9_dataset_builder._SIZE = 5
  qm9_dataset_builder._CHARACTERIZED_SIZE = 4

  DATASET_CLASS = qm9_dataset_builder.Builder

  DL_EXTRACT_RESULT = {
      'atomref': 'atomref.txt',
      'uncharacterized': 'uncharacterized.txt',
      # This is empty so it returns the base directory (where the
      # extracted files for the testdata are).
      'dsgdb9nsd': '',
  }

  SPLITS = {
      'train': 4,
  }


if __name__ == '__main__':
  testing.test_main()



# coding=utf-8
# Copyright 2025 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""MathQA dataset."""

import json
import os
import re

from etils import epath
import tensorflow_datasets.public_api as tfds

_URL = 'https://math-qa.github.io/data/MathQA.zip'

_FEATURES = [
    'Problem',
    'Rationale',
    'options',
    'correct',
    'annotated_formula',
    'linear_formula',
    'category',
    'correct_option',
]


class Builder(tfds.core.GeneratorBasedBuilder):
  """DatasetBuilder for math_qa dataset."""

  VERSION = tfds.core.Version('1.0.0')
  RELEASE_NOTES = {
      '1.0.0': 'Initial release.',
  }

  def _info(self) -> tfds.core.DatasetInfo:
    """Returns the dataset metadata."""
    return self.dataset_info_from_configs(
        features=tfds.features.FeaturesDict(
            {k: tfds.features.Text() for k in _FEATURES}
        ),
        supervised_keys=None,
        homepage='https://math-qa.github.io/',
    )

  def _split_generators(self, dl_manager: tfds.download.DownloadManager):
    """Returns SplitGenerators."""
    path = dl_manager.download_and_extract(_URL)
    return {
        tfds.Split.TRAIN: self._generate_examples(
            os.path.join(path, 'train.json')
        ),
        tfds.Split.VALIDATION: self._generate_examples(
            os.path.join(path, 'dev.json')
        ),
        tfds.Split.TEST: self._generate_examples(
            os.path.join(path, 'test.json')
        ),
    }

  def _generate_examples(self, filename: str):
    """Yields examples."""
    with epath.Path(filename).open() as f:
      for i, ex in enumerate(json.load(f)):
        ex['correct_option'] = extract_answer_text(ex['options'], ex['correct'])
        yield i, ex


def extract_answer_text(options_text: str, answer_tag: str):
  """Extracts correct answer's text from all options.

  Args:
    options_text: all options as text in various format.
    answer_tag: correct answers tag a, b, c, ...

  Returns:
    parsed option text corresponding to the correct answer.
  """
  if options_text.startswith('[') and options_text.endswith(']'):
    options = eval(options_text)  # pylint: disable = eval-used
    options = [re.sub('[abcde] \\)', '', x).strip() for x in options]
  else:
    options = re.split('[abcde] \\)', options_text)
    if options[0]:
      raise ValueError(f'Expects first segment to be empty in {options}.')
    options = [x.strip().rstrip(',').strip() for x in options[1:]]
  correct_id = ord(answer_tag) - ord('a')
  if correct_id >= len(options):
    raise ValueError(f'Ill parsed dictionary {options} from {options_text}.')
  return options[correct_id]
